@@ -135,15 +135,15 @@ QPixmap ShareListViewItem::createPropertyPixmap()
 
   int numberOfPix = 4; // the max number of pixmaps to join
 
-  int w = 22; // Standard size of one pixmap
+  int w = 16; // Standard size of one pixmap
   int margin = 4; // Margin between pixmaps
-  int h = 22;
+  int h = 16;
 
   int totalWidth = (w+margin)*numberOfPix;
 
   QPixmap pix(totalWidth,h);
 
-  pix.fill();  // Fill with white
+  pix.fill(Qt::transparent);
 
   QPainter p(&pix);
 
@@ -366,7 +366,7 @@ void KcmSambaConf::initAdvancedTab()
       icon = il->loadIcon("network-server-database",KIconLoader::Small);
     else {
       icon = QPixmap(16,16);
-      icon.fill();
+      icon.fill(Qt::transparent);
     }
       //SmallIcon("empty2");
 
@@ -740,7 +740,6 @@ void KcmSambaConf::loadLogging(SambaShare* )
   _dictMngr->add("debug hires timestamp",_interface->microsecondsChk);
   _dictMngr->add("syslog only",_interface->syslogOnlyChk);
   _dictMngr->add("debug timestamp",_interface->timestampChk);
-  _dictMngr->add("use mmap",_interface->useMmapChk);
 
 
 }
@@ -985,7 +984,7 @@ void KcmSambaConf::loadWinbind(SambaShare* )
 
   _dictMngr->add("winbind cache time", _interface->winbindCacheTimeSpin);
   _dictMngr->add("acl compatibility",_interface->aclCompatibilityCombo,
-                 new QStringList(QStringList() << "auto" << "winnt" << "win2k" << ""));
+                 new QStringList(QStringList() << "winnt" << "win2k" << "auto"));
 
   _dictMngr->add("winbind enum users",_interface->winbindEnumUsersChk);
   _dictMngr->add("winbind enum groups",_interface->winbindEnumGroupsChk);
@@ -1240,7 +1239,7 @@ void KcmSambaConf::addSambaUserBtnClicked()
     int passResult = K3PasswordDialog::getNewPassword(this,password,
                         i18n("<qt>Please enter a password for the user <b>%1</b></qt>", user.name));
     if (passResult != K3PasswordDialog::Accepted) {
-       list.remove(item);
+       list.removeAll(item);
        continue;
     }
 
@@ -1259,7 +1258,7 @@ void KcmSambaConf::addSambaUserBtnClicked()
       sambaItem->setDisabled(COL_NOPASSWORD, true);
 
 
-    list.remove(item);
+    list.removeAll(item);
     delete item;
   }
 }
@@ -1280,7 +1279,7 @@ void KcmSambaConf::removeSambaUserBtnClicked()
     }
 
     new K3ListViewItem(_interface->unixUsersListView, item->text(0), item->text(1));
-    list.remove(item);
+    list.removeAll(item);
     delete item;
   }
 }
