@@ -40,8 +40,8 @@
 #include <KSharedConfig>
 #include <KPluginInfo>
 #include <KComboBox>
-#include <KPassivePopup>
 #include <KConfigDialog>
+#include <KWindowSystem>
 
 #include <QTextStream>
 #include <QDomElement>
@@ -52,12 +52,28 @@
 
 #ifdef HAVE_NEPOMUK
     #include <Nepomuk/ResourceManager>
+    #include "nepomukcontroller.h"
 #endif
 
 #ifdef HAVE_KWORKSPACE
+    #include <QDBusConnection>
+    #include <QDBusInterface>
+    #include <QDBusPendingCall>
     #include <kworkspace/kworkspace.h>
+    #include <solid/control/powermanager.h>
+    #include <solid/powermanagement.h>
 #endif
 
+
+KGet::TransferData::TransferData(const KUrl &source, const KUrl &destination, const QString& group, bool doStart, const QDomElement *element)
+  : src(source),
+    dest(destination),
+    groupName(group),
+    start(doStart),
+    e(element)
+{
+}
+
 /**
  * This is our KGet class. This is where the user's transfers and searches are
  * stored and organized.
@@ -95,28 +111,21 @@ bool KGet::addGroup(const QString& groupName)
     return true;
 }
 
-void KGet::delGroup(TransferGroupHandler * group, bool askUser)
+void KGet::delGroup(TransferGroupHandler *group, bool askUser)
 {
-    TransferGroup * g = group->m_group;
-    if (group)
-    {
-        if (askUser) {
-            QWidget * configDialog = KConfigDialog::exists("preferences");
-            if (KMessageBox::warningYesNo(configDialog ? configDialog : m_mainWindow,
-                           i18n("Are you sure that you want to remove the group named %1?", g->name()),
-                           i18n("Remove Group"),
-                           KStandardGuiItem::remove(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
-                return;
-        }
-        JobQueue::iterator begin = g->begin();
-        JobQueue::iterator end = g->end();
-        for (; begin != end; begin++) {
-            Transfer *transfer = static_cast<Transfer*>(*begin);
-            m_transferTreeModel->delTransfer(transfer);//Unregister transfers from model first, they will get deleted by the group when destructing
-        }
-        m_transferTreeModel->delGroup(g);
-        g->deleteLater();
+    TransferGroup *g = group->m_group;
+
+    if (askUser) {
+        QWidget *configDialog = KConfigDialog::exists("preferences");
+        if (KMessageBox::warningYesNo(configDialog ? configDialog : m_mainWindow,
+                        i18n("Are you sure that you want to remove the group named %1?", g->name()),
+                        i18n("Remove Group"),
+                        KStandardGuiItem::remove(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
+            return;
     }
+
+    m_transferTreeModel->delGroup(g);
+    g->deleteLater();
 }
 
 void KGet::delGroups(QList<TransferGroupHandler*> groups, bool askUser)
@@ -240,22 +249,32 @@ TransferHandler * KGet::addTransfer(KUrl srcUrl, QString destDir, QString sugges
 
     if (destUrl == KUrl())
         return 0;
-    
-    return createTransfer(srcUrl, destUrl, groupName, start);
+
+    TransferHandler *transfer = createTransfer(srcUrl, destUrl, groupName, start);
+    if (transfer) {
+        KGet::showNotification(m_mainWindow, "added",
+                                i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                "kget", i18n("Download added"));
+    }
+
+    return transfer;
 }
 
-TransferHandler * KGet::addTransfer(const QDomElement& e, const QString& groupName)
+QList<TransferHandler*> KGet::addTransfers(const QList<QDomElement> &elements, const QString &groupName)
 {
-    //We need to read these attributes now in order to know which transfer
-    //plugin to use.
-    KUrl srcUrl = KUrl( e.attribute("Source") );
-    KUrl destUrl = KUrl( e.attribute("Dest") );
+    QList<TransferData> data;
 
-    kDebug(5001) << " src= " << srcUrl.url()
-              << " dest= " << destUrl.url() 
-              << " group= "<< groupName << endl;
+    foreach(const QDomElement &e, elements) {
+        //We need to read these attributes now in order to know which transfer
+        //plugin to use.
+        KUrl srcUrl = KUrl(e.attribute("Source"));
+        KUrl destUrl = KUrl(e.attribute("Dest"));
+        data << TransferData(srcUrl, destUrl, groupName, false, &e);
 
-    return createTransfer(srcUrl, destUrl, groupName, false, &e);
+        kDebug(5001) << "src=" << srcUrl << " dest=" << destUrl << " group=" << groupName;
+    }
+
+    return createTransfers(data);
 }
 
 const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString destDir, QString groupName, bool start)
@@ -297,6 +316,7 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
     it = urlsToDownload.begin();
     itEnd = urlsToDownload.end();
 
+    QList<TransferData> data;
     for ( ; it != itEnd; ++it )
     {
         if (destDir.isEmpty())
@@ -312,29 +332,45 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
         if (destUrl == KUrl())
             continue;
 
-        TransferHandler * newTransfer = createTransfer(*it, destUrl, groupName, start);
-        
-        if(newTransfer)
-            addedTransfers.append(newTransfer);
+        data << TransferData(*it, destUrl, groupName, start);
     }
-    
-    return addedTransfers;
+
+    QList<TransferHandler*> transfers = createTransfers(data);
+    foreach (TransferHandler *transfer, transfers) {
+        KGet::showNotification(m_mainWindow, "added",
+                                i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                "kget", i18n("Download added"));
+    }
+    return transfers;
 }
 
 
 bool KGet::delTransfer(TransferHandler * transfer)
 {
-    Transfer * t = transfer->m_transfer;
-    t->stop();
-    TransferHistoryStore::getStore()->saveItem(TransferHistoryItem(*t));
+    return delTransfers(QList<TransferHandler*>() << transfer);
+}
 
-    // TransferHandler deinitializations
-    transfer->destroy();
-    // Transfer deinitializations (the deinit function is called by the destroy() function)
-    t->destroy();
-    
-    m_transferTreeModel->delTransfer(t);
-    delete t;
+bool KGet::delTransfers(QList<TransferHandler*> handlers)
+{
+    if (!m_store) {
+        m_store = TransferHistoryStore::getStore();
+    }
+    QList<Transfer*> transfers;
+    QList<TransferHistoryItem> historyItems;
+    foreach (TransferHandler *handler, handlers) {
+        Transfer *transfer = handler->m_transfer;
+        transfers << transfer;
+        historyItems << TransferHistoryItem(*transfer);
+
+        // TransferHandler deinitializations
+        handler->destroy();
+        // Transfer deinitializations (the deinit function is called by the destroy() function)
+        transfer->destroy();
+    }
+    m_store->saveItems(historyItems);
+
+    m_transferTreeModel->delTransfers(transfers);
+    qDeleteAll(transfers);
     return true;
 }
 
@@ -362,6 +398,8 @@ QList<TransferHandler *> KGet::selectedTransfers()
     QList<TransferHandler *> selectedTransfers;
 
     QModelIndexList selectedIndexes = m_selectionModel->selectedRows();
+    //sort the indexes as this can speed up operations like deleting etc.
+    qSort(selectedIndexes.begin(), selectedIndexes.end());
 
     foreach(const QModelIndex &currentIndex, selectedIndexes)
     {
@@ -432,15 +470,16 @@ TransferTreeSelectionModel * KGet::selectionModel()
     return m_selectionModel;
 }
 
-void KGet::addTransferView(QAbstractItemView * view)
+#ifdef HAVE_NEPOMUK
+NepomukController *KGet::nepomukController()
 {
-    view->setModel(m_transferTreeModel);
-}
+    if (!m_nepomukController) {
+        m_nepomukController = new NepomukController;
+    }
 
-void KGet::addTransferView(KComboBox * view)
-{
-    view->setModel(m_transferTreeModel);
+    return m_nepomukController;
 }
+#endif
 
 void KGet::load( QString filename ) // krazy:exclude=passbyvalue
 {
@@ -519,10 +558,8 @@ void KGet::save( QString filename, bool plain ) // krazy:exclude=passbyvalue
     if ( !file.open( QIODevice::WriteOnly ) )
     {
         //kWarning(5001)<<"Unable to open output file when saving";
-        KMessageBox::error(0,
-                        i18n("Unable to save to: %1", filename),
-                        i18n("Error"));
-
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Unable to save to: %1", filename));
         return;
     }
 
@@ -582,6 +619,11 @@ bool KGet::schedulerRunning()
     return (m_scheduler->countRunningJobs() > 0);
 }
 
+void KGet::setSuspendScheduler(bool isSuspended)
+{
+    m_scheduler->setIsSuspended(isSuspended);
+}
+
 QList<TransferHandler*> KGet::allTransfers()
 {
     QList<TransferHandler*> transfers;
@@ -672,7 +714,7 @@ QList<TransferGroupHandler*> KGet::groupsFromExceptions(const KUrl &filename)
             }
 
             //now try with wildcards
-            if (!regExp.pattern().isEmpty() && !regExp.pattern().startsWith('*')) {
+            if (!regExp.pattern().isEmpty() && !regExp.pattern().contains('*')) {
                 regExp.setPattern('*' + regExp.pattern());
             }
 
@@ -724,6 +766,10 @@ QList<TransferFactory *> KGet::m_transferFactories;
 TransferGroupScheduler * KGet::m_scheduler = new TransferGroupScheduler();
 MainWindow * KGet::m_mainWindow = 0;
 KUiServerJobs * KGet::m_jobManager = 0;
+TransferHistoryStore * KGet::m_store = 0;
+#ifdef HAVE_NEPOMUK
+    NepomukController *KGet::m_nepomukController = 0;
+#endif
 
 // ------ PRIVATE FUNCTIONS ------
 KGet::KGet()
@@ -735,10 +781,10 @@ KGet::KGet()
     m_transferTreeModel = new TransferTreeModel(m_scheduler);
     m_selectionModel = new TransferTreeSelectionModel(m_transferTreeModel);
 
-    QObject::connect(m_transferTreeModel, SIGNAL(transferAddedEvent(TransferHandler *, TransferGroupHandler *)),
-                     m_jobManager,        SLOT(slotTransferAdded(TransferHandler *, TransferGroupHandler *)));
-    QObject::connect(m_transferTreeModel, SIGNAL(transferAboutToBeRemovedEvent(TransferHandler *, TransferGroupHandler *)),
-                     m_jobManager,        SLOT(slotTransferAboutToBeRemoved(TransferHandler *, TransferGroupHandler *)));
+    QObject::connect(m_transferTreeModel, SIGNAL(transfersAddedEvent(QList<TransferHandler*>)),
+                     m_jobManager,        SLOT(slotTransfersAdded(QList<TransferHandler*>)));
+    QObject::connect(m_transferTreeModel, SIGNAL(transfersAboutToBeRemovedEvent(QList<TransferHandler*>)),
+                     m_jobManager,        SLOT(slotTransfersAboutToBeRemoved(QList<TransferHandler*>)));
     QObject::connect(m_transferTreeModel, SIGNAL(transfersChangedEvent(QMap<TransferHandler *, Transfer::ChangesFlags>)),
                      m_jobManager,        SLOT(slotTransfersChanged(QMap<TransferHandler *, Transfer::ChangesFlags>)));
             
@@ -755,46 +801,73 @@ KGet::~KGet()
     delete m_transferTreeModel;
     delete m_jobManager;  //This one must always be before the scheduler otherwise the job manager can't remove the notifications when deleting.
     delete m_scheduler;
+    delete m_store;
+
+#ifdef HAVE_NEPOMUK
+    delete m_nepomukController;
+#endif
 }
 
 TransferHandler * KGet::createTransfer(const KUrl &src, const KUrl &dest, const QString& groupName, 
                           bool start, const QDomElement * e)
 {
-    kDebug(5001) << "srcUrl= " << src.url() << "  " 
-                             << "destUrl= " << dest.url() 
-                             << "group= _" << groupName << "_" << endl;
+    QList<TransferHandler*> transfer = createTransfers(QList<TransferData>() << TransferData(src, dest, groupName, start, e));
+    return (transfer.isEmpty() ? 0 : transfer.first());
+}
 
-    TransferGroup * group = m_transferTreeModel->findGroup(groupName);
-    if (group==0)
-    {
-        kDebug(5001) << "KGet::createTransfer  -> group not found";
-        group = m_transferTreeModel->transferGroups().first();
+QList<TransferHandler*> KGet::createTransfers(const QList<TransferData> &dataItems)
+{
+    QList<TransferHandler*> handlers;
+    if (dataItems.isEmpty()) {
+        return handlers;
     }
 
-    Transfer * newTransfer;
-    foreach (TransferFactory *factory, m_transferFactories)
-    {
-        kDebug(5001) << "Trying plugin   n.plugins=" << m_transferFactories.size();
-        if((newTransfer = factory->createTransfer(src, dest, group, m_scheduler, e)))
-        {
-//             kDebug(5001) << "KGet::createTransfer   ->   CREATING NEW TRANSFER ON GROUP: _" << group->name() << "_";
-            newTransfer->create();
-            newTransfer->load(e);
-            m_transferTreeModel->addTransfer(newTransfer, group);
+    QList<bool> start;
+    QHash<TransferGroup*, QList<Transfer*> > groups;
 
-            if(start)
-                newTransfer->handler()->start();
+    foreach (const TransferData &data, dataItems) {
+        kDebug(5001) << "srcUrl=" << data.src << " destUrl=" << data.dest << " group=" << data.groupName;
 
-            return newTransfer->handler();
+        TransferGroup *group = m_transferTreeModel->findGroup(data.groupName);
+        if (!group) {
+            kDebug(5001) << "KGet::createTransfer  -> group not found";
+            group = m_transferTreeModel->transferGroups().first();
+        }
+
+        Transfer *newTransfer = 0;
+        foreach (TransferFactory *factory, m_transferFactories) {
+            kDebug(5001) << "Trying plugin   n.plugins=" << m_transferFactories.size();
+            if ((newTransfer = factory->createTransfer(data.src, data.dest, group, m_scheduler, data.e))) {
+    //             kDebug(5001) << "KGet::createTransfer   ->   CREATING NEW TRANSFER ON GROUP: _" << group->name() << "_";
+                newTransfer->create();
+                newTransfer->load(data.e);
+                handlers << newTransfer->handler();
+                groups[group] << newTransfer;
+                start << data.start;
+                break;
+            }
+        }
+        if (!newTransfer) {
+            KGet::showNotification(m_mainWindow, "error",
+                                   i18n("Unable to continue: KGet cannot download using this protocol."), "dialog-error", i18n("Protocol unsupported"));
+
+            kWarning(5001) << "Warning! No plugin found to handle" << data.src;
         }
     }
-    
-    KMessageBox::error(m_mainWindow, 
-                       i18n("Unable to continue: KGet cannot download using this protocol."),
-                       i18n("Protocol unsupported"));
 
-    kDebug(5001) << "Warning! No plugin found to handle the given url";
-    return 0;
+    //add the created transfers to the model and start them if specified
+    QHash<TransferGroup*, QList<Transfer*> >::const_iterator it;
+    QHash<TransferGroup*, QList<Transfer*> >::const_iterator itEnd = groups.constEnd();
+    for (it = groups.constBegin(); it != itEnd; ++it) {
+        KGet::model()->addTransfers(it.value(), it.key());
+    }
+    for (int i = 0; i < handlers.count(); ++i) {
+        if (start[i]) {
+            handlers[i]->start();
+        }
+    }
+
+    return handlers;//TODO implement error message if it is 0, or should the addTransfers stuff do that, in case if the numer of returned items does not match the number of sent data?
 }
 
 TransferDataSource * KGet::createTransferDataSource(const KUrl &src, const QDomElement &type, QObject *parent)
@@ -813,22 +886,12 @@ TransferDataSource * KGet::createTransferDataSource(const KUrl &src, const QDomE
 
 QString KGet::generalDestDir(bool preferXDGDownloadDir)
 {
-    QString dir;
+    QString dir = Settings::lastDirectory();
 
     if (preferXDGDownloadDir) {
         dir = KGlobalSettings::downloadPath();
     }
 
-    if (dir.isEmpty()) {
-        dir = Settings::lastDirectory();
-        if (dir.isEmpty()) {
-            dir = KGlobalSettings::desktopPath();
-            if (dir.isEmpty()) {
-                dir = QDir::homePath();
-            }
-        }
-    }
-
     return dir;
 }
 
@@ -874,23 +937,17 @@ KUrl KGet::destFileInputDialog(QString destDir, const QString& suggestedFileName
     if (destDir.isEmpty())
         destDir = generalDestDir();
 
-    // open the filedialog for confirmation
-    QPointer<KFileDialog> dlg = new KFileDialog(destDir, QString(), 0, 0);
-    dlg->setCaption(i18n("Save As"));
-    dlg->setOperationMode(KFileDialog::Saving);
-    dlg->setMode(KFile::File | KFile::LocalOnly);
-
     // Use the destination name if not empty...
-    if (!suggestedFileName.isEmpty())
-        dlg->setSelection(suggestedFileName);
+    KUrl startLocation(destDir);
+    if (!suggestedFileName.isEmpty()) {
+        startLocation.addPath(suggestedFileName);
+    }
 
-    KUrl destUrl;
-    if (dlg->exec() != QDialog::Rejected){
-        destUrl = dlg->selectedUrl();
+    KUrl destUrl = KFileDialog::getSaveUrl(startLocation, QString(), m_mainWindow, i18n("Save As"));
+    if (!destUrl.isEmpty()) {
         Settings::setLastDirectory(destUrl.directory(KUrl::ObeyTrailingSlash));
     }
 
-    delete dlg;
     return destUrl;
 }
 
@@ -898,18 +955,15 @@ bool KGet::isValidSource(const KUrl &source)
 {
     // Check if the URL is well formed
     if (!source.isValid()) {
-        KMessageBox::error(0,
-                           i18n("Malformed URL:\n%1", source.prettyUrl()),
-                           i18n("Error"));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Malformed URL:\n%1", source.prettyUrl()));
 
         return false;
     }
     // Check if the URL contains the protocol
     if (source.protocol().isEmpty()){
-
-        KMessageBox::error(0, 
-                           i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()),
-                           i18n("Error"));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()));
 
         return false;
     }
@@ -1109,8 +1163,9 @@ KGetPlugin * KGet::createPluginFromService( const KService::Ptr &service )
 
     if (!factory)
     {
-        KMessageBox::error(0, i18n("<html><p>Plugin loader could not load the plugin:<br/><i>%1</i></p></html>",
-                                   service->library()));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Plugin loader could not load the plugin: %1.", service->library()),
+                               "dialog-info");
         kError(5001) << "KPluginFactory could not load the plugin:" << service->library();
         return 0;
     }
@@ -1126,7 +1181,7 @@ bool KGet::safeDeleteFile( const KUrl& url )
         QFileInfo info( url.toLocalFile() );
         if ( info.isDir() )
         {
-            KGet::showNotification(m_mainWindow, KNotification::Notification,
+            KGet::showNotification(m_mainWindow, "notification",
                                    i18n("Not deleting\n%1\nas it is a directory.", url.prettyUrl()),
                                    "dialog-info");
             return false;
@@ -1136,28 +1191,29 @@ bool KGet::safeDeleteFile( const KUrl& url )
     }
 
     else
-        KGet::showNotification(m_mainWindow, KNotification::Notification,
+        KGet::showNotification(m_mainWindow, "notification",
                                i18n("Not deleting\n%1\nas it is not a local file.", url.prettyUrl()),
                                "dialog-info");
     return false;
 }
 
-void KGet::showNotification(QWidget *parent, KNotification::StandardEvent eventId, 
-                            const QString &text, const QString &icon)
+KNotification *KGet::showNotification(QWidget *parent, const QString &eventType,
+                            const QString &text, const QString &icon, const QString &title, const KNotification::NotificationFlags &flags)
 {
-    KNotification::event(eventId, text, KIcon(icon).pixmap(KIconLoader::Small), parent);
+    return KNotification::event(eventType, title, text, KIcon(icon).pixmap(KIconLoader::SizeMedium), parent, flags);
 }
 
 GenericObserver::GenericObserver(QObject *parent)
   : QObject(parent),
-    m_save(0)
+    m_save(0),
+    m_finishAction(0)
 {
     connect(KGet::model(), SIGNAL(groupRemovedEvent(TransferGroupHandler*)), SLOT(groupRemovedEvent(TransferGroupHandler*)));
-    connect(KGet::model(), SIGNAL(transferAddedEvent(TransferHandler*, TransferGroupHandler*)), 
-                           SLOT(transferAddedEvent(TransferHandler*, TransferGroupHandler*)));
+    connect(KGet::model(), SIGNAL(transfersAddedEvent(QList<TransferHandler*>)),
+                           SLOT(transfersAddedEvent(QList<TransferHandler*>)));
     connect(KGet::model(), SIGNAL(groupAddedEvent(TransferGroupHandler*)), SLOT(groupAddedEvent(TransferGroupHandler*)));
-    connect(KGet::model(), SIGNAL(transferRemovedEvent(TransferHandler*, TransferGroupHandler*)), 
-                           SLOT(transferRemovedEvent(TransferHandler*, TransferGroupHandler*)));
+    connect(KGet::model(), SIGNAL(transfersRemovedEvent(QList<TransferHandler*>)),
+                           SLOT(transfersRemovedEvent(QList<TransferHandler*>)));
     connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)), 
                            SLOT(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)));
     connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags>)), 
@@ -1182,19 +1238,17 @@ void GenericObserver::groupRemovedEvent(TransferGroupHandler *handler)
     KGet::save();
 }
 
-void GenericObserver::transferAddedEvent(TransferHandler *handler, TransferGroupHandler *group)
+void GenericObserver::transfersAddedEvent(const QList<TransferHandler*> &handlers)
 {
-    Q_UNUSED(handler)
-    Q_UNUSED(group)
+    Q_UNUSED(handlers)
     requestSave();
     KGet::calculateGlobalSpeedLimits();
     KGet::checkSystemTray();
 }
 
-void GenericObserver::transferRemovedEvent(TransferHandler *handler, TransferGroupHandler *group)
+void GenericObserver::transfersRemovedEvent(const QList<TransferHandler*> &handlers)
 {
-    Q_UNUSED(handler)
-    Q_UNUSED(group)
+    Q_UNUSED(handlers)
     requestSave();
     KGet::calculateGlobalSpeedLimits();
     KGet::checkSystemTray();
@@ -1237,6 +1291,18 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
     {
         TransferHandler::ChangesFlags transferFlags = *it;
         TransferHandler *transfer = it.key();
+        
+        if (transferFlags & Transfer::Tc_Status) {
+            if ((transfer->status() == Job::Finished)   && (transfer->startStatus() != Job::Finished)) {
+                KGet::showNotification(KGet::m_mainWindow, "finished",
+                                       i18n("<p>The following file has finished downloading:</p><p style=\"font-size: small;\">\%1</p>", transfer->dest().fileName()),
+                                       "kget", i18n("Download completed"));
+            } else if (transfer->status() == Job::Running) {
+                KGet::showNotification(KGet::m_mainWindow, "started",
+                                       i18n("<p>The following transfer has been started:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                       "kget", i18n("Download started"));
+            }
+        }
 
         if (transferFlags & Transfer::Tc_Status)
             checkSysTray = true;
@@ -1260,21 +1326,55 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
             allFinished = false;
         }
     }
+    allFinished = allFinished && allTransfersFinished();
 
     if (checkSysTray)
         KGet::checkSystemTray();
 
-    if (Settings::afterFinishActionEnabled())
+    if (Settings::afterFinishActionEnabled() && allFinished)
     {
-        if (allFinished)
-            kDebug() << "All finished";
-        if (allFinished && Settings::afterFinishAction() == KGet::Quit)
-            checkAndFinish();
+        kDebug(5001) << "All finished";
+        KNotification *notification = 0;
 
+        if (!m_finishAction) {
+            m_finishAction = new QTimer(this);
+            m_finishAction->setSingleShot(true);
+            m_finishAction->setInterval(10000);
+            connect(m_finishAction, SIGNAL(timeout()), this, SLOT(slotAfterFinishAction()));
+        }
+
+        switch (Settings::afterFinishAction()) {
+            case KGet::Quit:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("KGet is now closing, as all downloads have completed."), "kget", "KGet", KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
 #ifdef HAVE_KWORKSPACE
-        if (allFinished && Settings::afterFinishAction() == KGet::Shutdown)
-            checkAndShutdown();
+            case KGet::Shutdown:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now turn off, as all downloads have completed."), "system-shutdown", i18nc("Shutting down computer", "Shutdown"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
+            case KGet::Hibernate:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now suspend to disk, as all downloads have completed."), "system-suspend-hibernate", i18nc("Hibernating computer", "Hibernating"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
+            case KGet::Suspend:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now suspend to RAM, as all downloads have completed."), "system-suspend", i18nc("Suspending computer", "Suspending"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
 #endif
+            default:
+                break;
+        }
+
+        if (notification) {
+            notification->setActions(QStringList() << i18nc("abort the proposed action", "Abort"));
+            connect(notification, SIGNAL(action1Activated()), this, SLOT(slotAbortAfterFinishAction()));
+            connect(m_finishAction, SIGNAL(timeout()), notification, SLOT(close()));
+
+            if (!m_finishAction->isActive()) {
+                m_finishAction->start();
+            }
+        }
+    } else if (allFinished) {
+        KGet::showNotification(KGet::m_mainWindow, "finishedall",
+                               i18n("<p>All transfers have been finished.</p>"),
+                               "kget", i18n("Downloads completed"));
     }
 }
 
@@ -1327,51 +1427,45 @@ bool GenericObserver::allTransfersFinished()
     return quitFlag;
 }
 
-KPassivePopup* GenericObserver::popupMessage(const QString &title, const QString &message)
+void GenericObserver::slotAfterFinishAction()
 {
-    KPassivePopup *popup;
-    // we have to call diferent message from kpassivePopup
-    // one with parent as QWidget for the mainWindow
-    // and another with parent as QSystemTrayIcon if the parent is a systemTray
-    // so passing the QSystemTrayIcon as QWidget don't work
-    if(Settings::enableSystemTray()) 
-    {
-        popup = KPassivePopup::message(5000, title, message, KGet::m_mainWindow);
-    }
-    else 
-    {
-        popup = KPassivePopup::message(5000, title, message, KGet::m_mainWindow);
-    }
+    kDebug(5001);
 
-    return popup;
-}
-
-void GenericObserver::checkAndFinish()
-{
-    kDebug();
-    // check if there is some unfinished transfer in scheduler queues
-    if(allTransfersFinished()) {
-        KPassivePopup *message = popupMessage(i18n("Quit KGet"),
-                                            i18n("KGet is now closing, as all downloads have completed."));
-        QObject::connect(message, SIGNAL(destroyed()), KGet::m_mainWindow, SLOT(slotQuit()));
+    switch (Settings::afterFinishAction()) {
+        case KGet::Quit:
+            kDebug(5001) << "Quit Kget.";
+            QTimer::singleShot(0, KGet::m_mainWindow, SLOT(slotQuit()));
+            break;
+    #ifdef HAVE_KWORKSPACE
+        case KGet::Shutdown:
+            QTimer::singleShot(0, KGet::m_mainWindow, SLOT(slotQuit()));
+            KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmNo,
+                        KWorkSpace::ShutdownTypeHalt,
+                        KWorkSpace::ShutdownModeForceNow);
+            break;
+        case KGet::Hibernate: {
+            QDBusConnection dbus(QDBusConnection::sessionBus());
+            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+            iface.asyncCall("suspend", Solid::Control::PowerManager::ToDisk);
+            break;
+        }
+        case KGet::Suspend: {
+            QDBusConnection dbus(QDBusConnection::sessionBus());
+            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+            iface.asyncCall("suspend", Solid::Control::PowerManager::ToRam);
+            break;
+        }
+    #endif
+        default:
+            break;
     }
 }
 
-#ifdef HAVE_KWORKSPACE
-void GenericObserver::checkAndShutdown()
+void GenericObserver::slotAbortAfterFinishAction()
 {
-    if(allTransfersFinished()) {
-        KPassivePopup *message = popupMessage(i18n("Quit KGet"),
-                                            i18n("The computer will now turn off, as all downloads have completed."));
-        QObject::connect(message, SIGNAL(destroyed()), SLOT(slotShutdown()));
-        QObject::connect(message, SIGNAL(destroyed()),  KGet::m_mainWindow, SLOT(slotQuit()));
-    }
+    kDebug(5001);
+
+    m_finishAction->stop();
 }
 
-void GenericObserver::slotShutdown()
-{
-    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmNo,
-                                KWorkSpace::ShutdownTypeHalt,
-                                KWorkSpace::ShutdownModeForceNow);
-}
-#endif
+#include "kget.moc"
