@@ -49,9 +49,10 @@ TransfersView::TransfersView(QWidget * parent)
     setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
 
-    populateHeaderActions();
     connect(header(), SIGNAL(customContextMenuRequested(const QPoint &)),
                       SLOT(slotShowHeaderMenu(const QPoint &)));
+    connect(header(), SIGNAL(sectionMoved(int,int,int)), this, SLOT(slotSectionMoved(int,int,int)));
+    connect(header(), SIGNAL(sectionResized(int,int,int)), this, SLOT(slotSaveHeader()));
     connect(this,     SIGNAL(doubleClicked(const QModelIndex &)),
             this,     SLOT(slotItemActivated(const QModelIndex &)));
     connect(this,     SIGNAL(collapsed(const QModelIndex &)),
@@ -62,54 +63,30 @@ TransfersView::TransfersView(QWidget * parent)
 
 TransfersView::~TransfersView()
 {
-    QList<int>  list;
-    for (int i = 0; i<5; i++)
-    {
-        int width = columnWidth(i);
-
-        if (Settings::columns().at(i) == 0) {
-            width = 90;
-        }
-
-        list.append(width);
-    }
-    Settings::setColumnWidths( list );
-    Settings::self()->writeConfig();
 }
 
 void TransfersView::setModel(QAbstractItemModel * model)
 {
     QTreeView::setModel(model);
-
     int nGroups = model->rowCount(QModelIndex());
 
     for(int i = 0; i < nGroups; i++)
     {
         kDebug(5001) << "openEditor for row " << i;
-        openPersistentEditor(model->index(i, 1, QModelIndex()));
+        openPersistentEditor(model->index(i, TransferTreeModel::Status, QModelIndex()));
     }
 
-    QList<int> sizeList = Settings::columnWidths();
-
-    if (!sizeList.isEmpty())
-    {
-        int j = 0;
-        foreach(int i, sizeList)
-        {
-            setColumnWidth( j, i );
-            j++;
-        }
-    }
-    else
-    {
+    QByteArray loadedState = QByteArray::fromBase64(Settings::headerState().toAscii());
+    if (loadedState.isNull()) {
         setColumnWidth(0 , 250);
+    } else {
+        header()->restoreState(loadedState);
     }
 
-    QList <int> columns = Settings::columns();
-    for (int i=0; i<columns.size(); i++) {
-        setColumnHidden(i, (columns.at(i) == 1) ? false : true);
-    }
+    //Workaround if the saved headerState is corrupted
+    header()->setRootIndex(QModelIndex());
 
+    populateHeaderActions();
     toggleMainGroup();
     connect(model, SIGNAL(rowsRemoved(const QModelIndex&, int, int)), SLOT (toggleMainGroup()));
 }
@@ -133,7 +110,7 @@ void TransfersView::rowsInserted(const QModelIndex & parent, int start, int end)
         for(int i = start; i <= end; i++)
         {
             kDebug(5001) << "openEditor for row " << i;
-            openPersistentEditor(model()->index(i, 1, parent));
+            openPersistentEditor(model()->index(i, TransferTreeModel::Status, parent));
         }
     }
 
@@ -148,21 +125,63 @@ void TransfersView::populateHeaderActions()
     m_headerMenu = new KMenu(header());
     m_headerMenu->addTitle(i18n("Select columns"));
 
-    QList <int> columns = Settings::columns();
     QSignalMapper *columnMapper = new QSignalMapper(this);
-    connect(columnMapper, SIGNAL(mapped(int)),
-                           SLOT(slotSetColumnVisible(int)));
+    connect(columnMapper, SIGNAL(mapped(int)), SLOT(slotHideSection(int)));
 
-    for(uint i=0; i<=TransferTreeModel::RemainingTime; i++) {
+    //Create for each column an action with the column-header as name
+    QVector<KAction*> orderedMenuItems(header()->count());
+    for (int i = 0; i < header()->count(); ++i) {
         KAction *action = new KAction(this);
-        action->setText(TransferTreeModel::columnName(i));
+        action->setText(model()->headerData(i, Qt::Horizontal).toString());
         action->setCheckable(true);
-        action->setChecked((columns.at(i) == 1) ? true : false);
-        m_headerMenu->addAction(action);
+        action->setChecked(!header()->isSectionHidden(i));
+        orderedMenuItems[header()->visualIndex(i)] = action;
 
         connect(action, SIGNAL(toggled(bool)), columnMapper, SLOT(map()));
         columnMapper->setMapping(action, i);
     }
+
+    //append the sorted actions
+    for (int i = 0; i < orderedMenuItems.count(); ++i) {
+        m_headerMenu->addAction(orderedMenuItems[i]);
+    }
+}
+
+void TransfersView::slotHideSection(int logicalIndex)
+{
+    const bool hide = !header()->isSectionHidden(logicalIndex);
+    header()->setSectionHidden(logicalIndex, hide);
+    slotSaveHeader();
+}
+
+void TransfersView::slotSectionMoved(int logicalIndex, int oldVisualIndex, int newVisualIndex)
+{
+    Q_UNUSED(logicalIndex)
+
+    //first item is the title, so increase the indexes by one
+    ++oldVisualIndex;
+    ++newVisualIndex;
+    QList<QAction*> actions = m_headerMenu->actions();
+
+    QAction *before = actions.last();
+    if (newVisualIndex + 1 < actions.count()) {
+        if (newVisualIndex > oldVisualIndex) {
+            before = actions[newVisualIndex + 1];
+        } else {
+            before = actions[newVisualIndex];
+        }
+    }
+
+    QAction *action = actions[oldVisualIndex];
+    m_headerMenu->removeAction(action);
+    m_headerMenu->insertAction(before, action);
+    slotSaveHeader();
+}
+
+void TransfersView::slotSaveHeader()
+{
+    Settings::setHeaderState(header()->saveState().toBase64());
+    Settings::self()->writeConfig();
 }
 
 void TransfersView::dragMoveEvent ( QDragMoveEvent * event )
@@ -184,8 +203,8 @@ void TransfersView::slotItemActivated(const QModelIndex & index)
 
     if(!item)
         return;
-    
-    if(!item->isGroup() && Settings::showExpandableTransferDetails() && index.column() == 0) {
+
+    if(!item->isGroup() && index.column() == 0) {
         if(!view_delegate->isExtended(index)) {
             TransferHandler *handler = item->asTransfer()->transferHandler();
             QWidget *widget = getDetailsWidgetForTransfer(handler);
@@ -249,23 +268,6 @@ void TransfersView::rowsAboutToBeRemoved(const QModelIndex & parent, int start,
     closeExpandableDetails(currentIndex());
 }
 
-void TransfersView::slotSetColumnVisible(int column)
-{
-    QList <int> columns = Settings::columns();
-
-    if (columns.size() >= column) {
-        columns.replace(column, (columns.at(column) == 1) ? 0 : 1);
-    }
-    else {
-        columns.insert(column, 0);
-    }
-
-    setColumnHidden(column, (columns.at(column) == 1) ? false : true);
-
-    Settings::setColumns(columns);
-    Settings::self()->writeConfig();
-}
-
 void TransfersView::slotShowHeaderMenu(const QPoint &point)
 {
     m_headerMenu->popup(header()->mapToGlobal(point));
@@ -287,10 +289,12 @@ void TransfersView::closeExpandableDetails(const QModelIndex &transferIndex)
 void TransfersView::selectionChanged(const QItemSelection & selected, const QItemSelection & deselected)
 {
     Q_UNUSED(deselected)
-    if (selected.indexes().isEmpty())
-        return;
-    TransfersViewDelegate *view_delegate = static_cast<TransfersViewDelegate *>(itemDelegate());
-    KGet::actionCollection()->action("transfer_show_details")->setChecked(view_delegate->isExtended(selected.indexes().first()));
+    if (!selected.indexes().isEmpty()) {
+        TransfersViewDelegate *view_delegate = static_cast<TransfersViewDelegate *>(itemDelegate());
+        KGet::actionCollection()->action("transfer_show_details")->setChecked(view_delegate->isExtended(selected.indexes().first()));
+    }
+
+    QTreeView::selectionChanged(selected, deselected);
 }
 
 void TransfersView::closeExpandableDetails(const QModelIndex &parent, int rowStart, int rowEnd)
