@@ -44,6 +44,12 @@
 #include <QtGui/QScrollBar>
 #include <QMimeData>
 #include <QApplication>
+#include <QFileDialog>
+
+#include <Phonon/MediaObject>
+#include <Phonon/Path>
+#include <Phonon/AudioOutput>
+#include <Phonon/Global>
 
 // KHTML::DOM includes
 #include <dom/dom_doc.h>
@@ -95,6 +101,8 @@
 #include "kopetechatwindowstyle.h"
 #include "kopetechatwindowstylemanager.h"
 
+static const uint ConsecutiveMessageTimeout = 15 /* minutes */ * 60 /* (seconds/minute) */;
+
 class ToolTip;
 
 class ChatMessagePart::Private
@@ -105,7 +113,7 @@ public:
 	   copyAction(0), saveAction(0), printAction(0),
 	   closeAction(0),copyURLAction(0), currentChatStyle(0),
 	   latestDirection(Kopete::Message::Inbound), latestType(Kopete::Message::TypeNormal),
-	   htmlEventListener(0)
+	   latestTime(0), htmlEventListener(0)
 	{}
 
 	~Private()
@@ -131,11 +139,11 @@ public:
 	KAction *closeAction;
 	KAction *copyURLAction;
 
-	// We can't use QPointer because ChatWindowStyle is not a QObject
-	ChatWindowStyle *currentChatStyle;
+	ChatWindowStyle* currentChatStyle;
 	QPointer<Kopete::Contact> latestContact;
 	Kopete::Message::MessageDirection latestDirection;
 	Kopete::Message::MessageType latestType;
+	uint latestTime;
 	// Yep I know it will take memory, but I don't have choice
 	// to enable on-the-fly style changing.
 	QList<Kopete::Message> allMessages;
@@ -207,10 +215,10 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 {
 	d->manager = mgr;
 
-	d->currentChatStyle = ChatWindowStyleManager::self()->getStyleFromPool(
-			 KopeteChatWindowSettings::self()->styleName() );
+	d->currentChatStyle = ChatWindowStyleManager::self()->getValidStyleFromPool( KopeteChatWindowSettings::self()->styleName() );
+	if (d->currentChatStyle)
+		connect( d->currentChatStyle, SIGNAL(destroyed(QObject*)), this, SLOT(clearStyle()) );
 
-	kDebug(14000) << d->currentChatStyle->getStyleName();
 	connect( this, SIGNAL(completed()), this, SLOT(slotRenderingFinished()) );
 
 	//Security settings, we don't need this stuff
@@ -252,7 +260,10 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 
 	connect( this, SIGNAL(popupMenu(const QString &, const QPoint &)),
 	         this, SLOT(slotRightClick(const QString &, const QPoint &)) );
-	connect( view()->verticalScrollBar(), SIGNAL(sliderMoved(int)),
+
+	// setup scrollbar
+	view()->verticalScrollBar()->setTracking(true);
+	connect( view()->verticalScrollBar(), SIGNAL(valueChanged(int)),
 	         this, SLOT(slotScrollingTo(int)) );
 
 	connect( Kopete::TransferManager::transferManager(), SIGNAL(askIncomingDone(unsigned int)),
@@ -364,7 +375,7 @@ void ChatMessagePart::slotOpenURLRequest(const KUrl &url, const KParts::OpenUrlA
 	kDebug(14000) << "url=" << url.url();
 	if ( url.protocol() == QLatin1String("kopetemessage") )
 	{
-		Kopete::Contact *contact = d->manager->account()->contacts()[ url.host() ];
+		Kopete::Contact *contact = d->manager->account()->contacts().value( url.host() );
 		if ( contact )
 			contact->execute();
 	}
@@ -401,21 +412,27 @@ void ChatMessagePart::readOverrides()
 void ChatMessagePart::setStyle( const QString &styleName )
 {
 	// Create a new ChatWindowStyle
-	d->currentChatStyle = ChatWindowStyleManager::self()->getStyleFromPool(styleName);
+	setStyle( ChatWindowStyleManager::self()->getValidStyleFromPool(styleName) );
+}
+
+void ChatMessagePart::setStyle( ChatWindowStyle *style )
+{
+	// Change the current style
+	if (d->currentChatStyle)
+		disconnect( d->currentChatStyle, SIGNAL(destroyed(QObject*)), this, SLOT(clearStyle()) );
+
+	d->currentChatStyle = style;
+	if (d->currentChatStyle)
+		connect( d->currentChatStyle, SIGNAL(destroyed(QObject*)), this, SLOT(clearStyle()) );
 
 	// Do the actual style switch
 	// Wait for the event loop before switching the style
 	QTimer::singleShot( 0, this, SLOT(changeStyle()) );
 }
 
-void ChatMessagePart::setStyle( ChatWindowStyle *style )
+void ChatMessagePart::clearStyle()
 {
-	// Change the current style
-	d->currentChatStyle = style;
-
-	// Do the actual style switch
-	// Wait for the event loop before switching the style
-	QTimer::singleShot( 0, this, SLOT(changeStyle()) );
+	setStyle( QString() );
 }
 
 void ChatMessagePart::setStyleVariant( const QString &variantPath )
@@ -449,6 +466,9 @@ void ChatMessagePart::slotAppearanceChanged()
 
 void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 {
+	if ( !d->currentChatStyle )
+		return;
+
 	// Don't remove foreground color for history messages.
 	if ( !message.classes().contains("history") )
 	{
@@ -484,6 +504,14 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 		isConsecutiveMessage = (message.direction() == d->latestDirection && !d->latestContact.isNull()
 		                        && d->latestContact == message.from() && message.type() == d->latestType
 		                        && message.type() != Kopete::Message::TypeFileTransferRequest );
+
+		if(message.timestamp().isValid()){
+			uint next = message.timestamp().toTime_t();
+			if(isConsecutiveMessage && (next - d->latestTime) > ConsecutiveMessageTimeout){
+				isConsecutiveMessage = false;
+			}
+			d->latestTime = next;
+		}
 	}
 
 	// Don't test it in the switch to don't break consecutive messages.
@@ -514,6 +542,10 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 	{
 		formattedMessageHtml = d->currentChatStyle->getFileTransferIncomingHtml();
 	}
+	else if(message.type() == Kopete::Message::TypeVoiceClipRequest)
+	{
+		formattedMessageHtml = d->currentChatStyle->getVoiceClipIncomingHtml();
+	}
 	else
 	{
 		switch(message.direction())
@@ -586,6 +618,10 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 		else
 			addFileTransferButtonsEventListener( message.id() );
 	}
+	else if ( message.type() == Kopete::Message::TypeVoiceClipRequest )
+	{
+		addVoiceClipsButtonsEventListener( message.id() );
+	}
 
 	// Keep the direction to see on next message
 	// if it's a consecutive message
@@ -908,7 +944,8 @@ void ChatMessagePart::khtmlDrawContentsEvent( khtml::DrawContentsEvent * event)
 
 void ChatMessagePart::slotCloseView( bool force )
 {
-	d->manager->view()->closeView( force );
+	if (d->manager && d->manager->view())
+		d->manager->view()->closeView( force );
 }
 
 void ChatMessagePart::emitTooltipEvent(  const QString &textUnderMouse, QString &toolTip )
@@ -919,6 +956,9 @@ void ChatMessagePart::emitTooltipEvent(  const QString &textUnderMouse, QString
 // Style formatting for messages(incoming, outgoing, status)
 QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const Kopete::Message &_message )
 {
+	if ( !d->currentChatStyle )
+		return QString();
+
 	Kopete::Message message=_message; //we will eventually need to modify it before showing it.
 	QString resultHTML = sourceHTML;
 	QString nick, contactId, service, protocolIcon, nickLink;
@@ -1087,6 +1127,19 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 		resultHTML.replace( QLatin1String("%cancelRequestHandlerId%"), QString( "ftCC%1" ).arg( message.id() ) );
 	}
 
+	if ( message.type() == Kopete::Message::TypeVoiceClipRequest )
+	{
+		QString fileIcon;
+
+		QString iconName = KMimeType::iconNameForUrl( message.fileName() );
+		fileIcon = KIconLoader::global()->iconPath( iconName, -KIconLoader::SizeMedium );
+
+		resultHTML.replace( QLatin1String("%fileIconPath%"), fileIcon );
+
+		resultHTML.replace( QLatin1String("%playVoiceHandlerId%"), QString( "vcPL%1" ).arg( message.id() ) );
+		resultHTML.replace( QLatin1String("%saveAsVoiceHandlerId%"), QString( "vcSA%1" ).arg( message.id() ) );
+	}
+
 	if ( message.type() == Kopete::Message::TypeNormal && message.direction() == Kopete::Message::Outbound )
 		resultHTML.replace( QLatin1String( "%stateElementId%" ), QString( "msST%1" ).arg( message.id() ) );
 
@@ -1159,7 +1212,7 @@ QString ChatMessagePart::formatTime(const QString &_timeFormat, const QDateTime
 	char buffer[256];
 #ifdef Q_WS_WIN
 	QString timeFormat = _timeFormat;
-	// %e is not supported on windows
+	// some formats are not supported on windows (gnu extension?)
 	timeFormat = timeFormat.replace(QLatin1String("%e"), QLatin1String("%d"));
 	timeFormat = timeFormat.replace(QLatin1String("%T"), QLatin1String("%H:%M:%S"));
 #else
@@ -1199,7 +1252,7 @@ QString ChatMessagePart::formatName( const Kopete::Contact* contact, Qt::TextFor
 		return QString();
 	}
 
-	// Use metacontact display name if the metacontact exists and if its not the myself metacontact.
+	// Use metacontact display name if the metacontact exists and if it is not the myself metacontact.
 	// Myself metacontact is not a reliable source.
 	if ( contact->metaContact() && contact->metaContact() != Kopete::ContactList::self()->myself() )
 	{
@@ -1285,37 +1338,44 @@ void ChatMessagePart::writeTemplate()
 	// baseStyle: Import the main.css from the Chat Window Style
 	// mainStyle: Currrent variant CSS url.
 
-	// FIXME: Maybe this string should be load from a file, then parsed for args.
 	QString xhtmlBase;
-	xhtmlBase += QString("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
-		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"\n"
-		"\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
-		"<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
-    	"<head>\n"
-        "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\n\" />\n"
-        "<base href=\"%1\">\n"
-		"<style id=\"KopeteStyle\" type=\"text/css\" media=\"screen,print\">\n"
-		"	%5\n"
-		"</style>\n"
-		"<style id=\"baseStyle\" type=\"text/css\" media=\"screen,print\">\n"
-		"	@import url(\"main.css\");\n"
-		"	*{ word-wrap:break-word; }\n"
-		"</style>\n"
-		"<style id=\"mainStyle\" type=\"text/css\" media=\"screen,print\">\n"
-		"	@import url(\"%4\");\n"
-        "</style>\n"
-		"</head>\n"
-		"<body>\n"
-		"%2\n"
-		"<div id=\"Chat\">\n</div>\n"
-		"%3\n"
-		"</body>"
-		"</html>"
-		).arg( d->currentChatStyle->getStyleBaseHref() )
-		.arg( formatStyleKeywords(d->currentChatStyle->getHeaderHtml()) )
-		.arg( formatStyleKeywords(d->currentChatStyle->getFooterHtml()) )
-		.arg( adjustStyleVariantForChatSession( KopeteChatWindowSettings::self()->styleVariant() ) )
-		.arg( styleHTML() );
+	if ( d->currentChatStyle )
+	{
+		// FIXME: Maybe this string should be load from a file, then parsed for args.
+		xhtmlBase += QString("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+			"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"\n"
+			"\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
+			"<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
+			"<head>\n"
+			"<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\n\" />\n"
+			"<base href=\"%1\">\n"
+			"<style id=\"KopeteStyle\" type=\"text/css\" media=\"screen,print\">\n"
+			"	%5\n"
+			"</style>\n"
+			"<style id=\"baseStyle\" type=\"text/css\" media=\"screen,print\">\n"
+			"	@import url(\"main.css\");\n"
+			"	*{ word-wrap:break-word; }\n"
+			"</style>\n"
+			"<style id=\"mainStyle\" type=\"text/css\" media=\"screen,print\">\n"
+			"	@import url(\"%4\");\n"
+			"</style>\n"
+			"</head>\n"
+			"<body>\n"
+			"%2\n"
+			"<div id=\"Chat\">\n</div>\n"
+			"%3\n"
+			"</body>"
+			"</html>"
+			).arg( d->currentChatStyle->getStyleBaseHref() )
+			.arg( formatStyleKeywords(d->currentChatStyle->getHeaderHtml()) )
+			.arg( formatStyleKeywords(d->currentChatStyle->getFooterHtml()) )
+			.arg( adjustStyleVariantForChatSession( KopeteChatWindowSettings::self()->styleVariant() ) )
+			.arg( styleHTML() );
+	}
+	else
+	{
+		xhtmlBase = i18n( "Chat style could not be found, or is invalid." );
+	}
 	write(xhtmlBase);
 	end();
 #ifdef STYLE_TIMETEST
@@ -1360,9 +1420,51 @@ void ChatMessagePart::resendMessage( uint messageId )
 	}
 }
 
+void ChatMessagePart::saveVoiceClip( uint messageId )
+{
+	QList<Kopete::Message>::ConstIterator it, itEnd = d->allMessages.constEnd();
+	for ( it = d->allMessages.constBegin(); it != itEnd; ++it )
+	{
+		if ( (*it).id() == messageId )
+		{
+			if(!(*it).fileName().isEmpty())
+			{
+				QString temporaryExtension = "*.wav";
+				QString newFileName = QFileDialog::getSaveFileName(NULL,
+						i18n("Save File as"), QString(), i18n("Wav file (*.wav)"), &temporaryExtension);
+				if(!newFileName.isEmpty())
+					QFile::copy((*it).fileName(), newFileName);
+			}
+			break;
+		}
+	}
+}
+
+void ChatMessagePart::playVoiceClip( uint messageId )
+{
+	QList<Kopete::Message>::ConstIterator it, itEnd = d->allMessages.constEnd();
+	for ( it = d->allMessages.constBegin(); it != itEnd; ++it )
+	{
+		if ( (*it).id() == messageId )
+		{
+			if(!(*it).fileName().isEmpty())
+			{
+				Phonon::MediaObject *media = new Phonon::MediaObject(this);
+				Phonon::AudioOutput *audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
+				connect(media, SIGNAL(finished()), media, SLOT(deleteLater()));
+				connect(media, SIGNAL(finished()), audioOutput, SLOT(deleteLater()));
+				createPath(media, audioOutput);
+				media->setCurrentSource((*it).fileName());
+				media->play();
+			}
+			break;
+		}
+	}
+}
+
 QString ChatMessagePart::adjustStyleVariantForChatSession( const QString & styleVariant ) const
 {
-	if ( d->manager->form() == Kopete::ChatSession::Chatroom
+	if ( d->currentChatStyle && d->manager->form() == Kopete::ChatSession::Chatroom
 			&& KopeteChatWindowSettings::self()->useCompact() ) {
 		return d->currentChatStyle->compact( styleVariant );
 	}
@@ -1399,6 +1501,16 @@ void ChatMessagePart::addFileTransferButtonsEventListener( unsigned int id )
 	registerClickEventListener( document().getElementById( elementId ) );
 }
 
+void ChatMessagePart::addVoiceClipsButtonsEventListener( unsigned int id )
+{
+	QString elementId = QString( "vcSA%1" ).arg( id );
+	registerClickEventListener( document().getElementById( elementId ) );
+
+	elementId = QString( "vcPL%1" ).arg( id );
+	registerClickEventListener( document().getElementById( elementId ) );
+}
+
+
 void ChatMessagePart::disableFileTransferButtons( unsigned int id )
 {
 	QString elementId = QString( "ftSV%1" ).arg( id );
@@ -1419,6 +1531,9 @@ void ChatMessagePart::disableFileTransferButtons( unsigned int id )
 
 void ChatMessagePart::changeMessageStateElement( uint id, Kopete::Message::MessageState state )
 {
+	if ( !d->currentChatStyle )
+		return;
+
 	QString elementId = QString( "msST%1" ).arg( id );
 	DOM::HTMLElement element = document().getElementById( elementId );
 	if ( element.isNull() )
@@ -1457,6 +1572,8 @@ void ChatMessagePart::registerClickEventListener( DOM::HTMLElement element )
 	{
 		d->htmlEventListener = new HTMLEventListener();
 		connect( d->htmlEventListener, SIGNAL(resendMessage(uint)), this, SLOT(resendMessage(uint)) );
+		connect( d->htmlEventListener, SIGNAL(playVoiceClip(uint)), this, SLOT(playVoiceClip(uint)) );
+		connect( d->htmlEventListener, SIGNAL(saveVoiceClip(uint)), this, SLOT(saveVoiceClip(uint)) );
 	}
 	element.addEventListener( "click", d->htmlEventListener, false );
 }
@@ -1486,6 +1603,10 @@ void HTMLEventListener::handleEvent( DOM::Event &event )
 			Kopete::TransferManager::transferManager()->cancelIncomingTransfer( messageId );
 		else if ( idType == QLatin1String( "msRS" ) )
 			emit resendMessage( messageId );
+		else if ( idType == QLatin1String( "vcPL" ) )
+			emit playVoiceClip( messageId );
+		else if ( idType == QLatin1String( "vcSA" ) )
+			emit saveVoiceClip( messageId );
 	}
 }
 
