@@ -1,6 +1,7 @@
 /*
     kopetechatwindow.cpp - Chat Window
 
+    Copyright (c) 2008      by Benson Tsai           <btsai@vrwarp.com>
     Copyright (c) 2007      by Gustavo Pichorim Boiko <gustavo.boiko@kdemail.net>
     Copyright (c) 2002-2006 by Olivier Goffart       <ogoffart@kde.org>
     Copyright (c) 2003-2004 by Richard Smith         <kde@metafoo.co.uk>
@@ -83,6 +84,7 @@
 
 #include <qtoolbutton.h>
 #include <kxmlguifactory.h>
+#include <KTabBar>
 
 typedef QMap<Kopete::Account*,KopeteChatWindow*> AccountMap;
 typedef QMap<Kopete::Group*,KopeteChatWindow*> GroupMap;
@@ -202,7 +204,7 @@ KopeteChatWindow::KopeteChatWindow( Kopete::ChatSession::Form form, QWidget *par
 	updateBg = true;
 	m_tabBar = 0L;
 
-	m_participantsWidget = new QDockWidget(this);
+	m_participantsWidget = new QDockWidget(i18n("Participants"), this);
 	m_participantsWidget->setAllowedAreas(Qt::RightDockWidgetArea | Qt::LeftDockWidgetArea);
 	m_participantsWidget->setFeatures(QDockWidget::DockWidgetClosable);
 	m_participantsWidget->setTitleBarWidget(0L);
@@ -283,26 +285,26 @@ KopeteChatWindow::~KopeteChatWindow()
 
 	for( AccountMap::Iterator it = accountMap.begin(); it != accountMap.end(); )
 	{
-		AccountMap::Iterator mayDeleteIt = it;
-		++it;
-		if( mayDeleteIt.value() == this )
-			accountMap.remove( mayDeleteIt.key() );
+		if( it.value() == this )
+			it=accountMap.erase( it );
+		else
+			++it;
 	}
 
 	for( GroupMap::Iterator it = groupMap.begin(); it != groupMap.end(); )
 	{
-		GroupMap::Iterator mayDeleteIt = it;
-		++it;
-		if( mayDeleteIt.value() == this )
-			groupMap.remove( mayDeleteIt.key() );
+		if( it.value() == this )
+			it=groupMap.erase( it );
+		else
+			++it;
 	}
 
 	for( MetaContactMap::Iterator it = mcMap.begin(); it != mcMap.end(); )
 	{
-		MetaContactMap::Iterator mayDeleteIt = it;
-		++it;
-		if( mayDeleteIt.value() == this )
-			mcMap.remove( mayDeleteIt.key() );
+		if( it.value() == this )
+			it=mcMap.erase( it );
+		else
+			++it;
 	}
 
 	windows.removeAt( windows.indexOf( this ) );
@@ -328,17 +330,17 @@ void KopeteChatWindow::slotTabContextMenu( QWidget *tab, const QPoint &pos )
 {
 	m_popupView = static_cast<ChatView*>( tab );
 
-	KMenu *popup = new KMenu;
-	popup->addTitle( KStringHandler::rsqueeze( m_popupView->caption() ) );
-	popup->addAction( actionContactMenu );
-	popup->addSeparator();
-	popup->addAction( actionTabPlacementMenu );
-	popup->addAction( tabDetach );
-	popup->addAction( actionDetachMenu );
-	popup->addAction( tabClose );
-	popup->exec( pos );
+	KMenu popup;
+	popup.addTitle( KStringHandler::rsqueeze( m_popupView->caption() ) );
+	popup.addAction( actionContactMenu );
+	popup.addSeparator();
+	popup.addAction( actionTabPlacementMenu );
+	popup.addAction( tabDetach );
+	popup.addAction( actionDetachMenu );
+	popup.addAction( tabCloseAllOthers );
+	popup.addAction( tabClose );
+	popup.exec( pos );
 
-	delete popup;
 	m_popupView = 0;
 }
 
@@ -347,6 +349,14 @@ ChatView *KopeteChatWindow::activeView()
 	return m_activeView;
 }
 
+void KopeteChatWindow::updateSendKeySequence()
+{
+	if ( !sendMessage || !m_activeView )
+		return;
+
+	m_activeView->editPart()->textEdit()->setSendKeySequenceList( sendMessage->shortcuts() );
+}
+
 void KopeteChatWindow::initActions(void)
 {
 	KActionCollection *coll = actionCollection();
@@ -354,7 +364,11 @@ void KopeteChatWindow::initActions(void)
 	createStandardStatusBarAction();
 
 	chatSend = new KAction( KIcon("mail-send"), i18n( "&Send Message" ), coll );
-	coll->addAction( "chat_send", chatSend );
+	//Recuperate the qAction for later
+	sendMessage = coll->addAction( "chat_send", chatSend );
+	//Set up change signal in case the user changer the shortcut later
+	connect( sendMessage, SIGNAL(changed()), SLOT(updateSendKeySequence()) );
+
 	connect( chatSend, SIGNAL( triggered(bool) ), SLOT( slotSendMessage() ) );
 	//Default to 'Return' and 'Enter' for sending messages
 	//'Return' is the key in the main part of the keyboard
@@ -363,7 +377,7 @@ void KopeteChatWindow::initActions(void)
 	chatSend->setShortcut( chatSendShortcut );
 	chatSend->setEnabled( false );
 
-	chatSendFile = new KAction( KIcon("mail-attachment"), i18n( "Send File" ), coll );
+	chatSendFile = new KAction( KIcon("mail-attachment"), i18n( "Send File..." ), coll );
 	coll->addAction( "chat_send_file", chatSendFile );
 	connect( chatSendFile, SIGNAL( triggered(bool) ), SLOT( slotSendFile() ) );
 	chatSendFile->setEnabled( false );
@@ -405,6 +419,11 @@ void KopeteChatWindow::initActions(void)
 	tabDetach->setEnabled( false );
 	connect( tabDetach, SIGNAL(triggered(bool)), this, SLOT( slotDetachChat() ));
 
+	tabCloseAllOthers = new KAction( KIcon("tab-close"), i18n( "Close &All But This Tab" ), coll );
+	coll->addAction( "tabs_close_others", tabCloseAllOthers );
+	tabCloseAllOthers->setEnabled( true );
+	connect( tabCloseAllOthers, SIGNAL(triggered(bool)), this, SLOT( slotCloseAllOtherTabs() ));
+
 	actionDetachMenu = new KActionMenu( KIcon("tab-detach"), i18n( "&Move Tab to Window" ), coll );
 	coll->addAction( "tabs_detachmove", actionDetachMenu );
 	actionDetachMenu->setDelayed( false );
@@ -424,25 +443,6 @@ void KopeteChatWindow::initActions(void)
 	KStandardAction::paste( this, SLOT(slotPaste()), coll);
 
 	KAction* action;
-	action = new KAction( KIcon("preferences-desktop-font"), i18n( "Set &Font..." ), coll );
-	coll->addAction( "format_font", action );
-	connect( action, SIGNAL(triggered(bool)), this, SLOT(slotSetFont()) );
-
-	action = new KAction( KIcon("format-stroke-color"), i18n( "Reset Font And Color" ), coll );
-	coll->addAction( "format_font_and_color_reset", action );
-	connect( action, SIGNAL(triggered(bool)), this, SLOT(slotResetFontAndColor()) );
-
-	action = new KAction( KIcon("format-text-color"), i18n( "Set Text &Color..." ), coll );
-	coll->addAction( "format_fgcolor", action );
-	connect( action, SIGNAL(triggered(bool)), this, SLOT(slotSetFgColor()) );
-
-	action = new KAction( KIcon("format-fill-color"), i18n( "Set &Background Color..." ), coll );
-	coll->addAction( "format_bgcolor", action );
-	connect( action, SIGNAL(triggered()), this, SLOT(slotSetBgColor()) );
-
-	toggleRichText = new KToggleAction( KIcon("draw-freehand"), i18n("Enable &Rich Text"), this );
-	coll->addAction( "enable_richtext", toggleRichText );
-	connect( toggleRichText, SIGNAL(toggled(bool)), this, SLOT(toggleRichTextAction(bool)) );
 
 	historyUp = new KAction( i18n( "Previous History" ), coll );
 	coll->addAction( "history_up", historyUp );
@@ -575,24 +575,9 @@ void KopeteChatWindow::enableSpellCheckAction(bool enable)
 	toggleAutoSpellCheck->setChecked( enable );
 }
 
-void KopeteChatWindow::updateRichTextAction()
-{
-	if ( !m_activeView )
-		return;
-
-	toggleRichText->setEnabled( m_activeView->editPart()->isRichTextAvailable() );
-	toggleRichText->setChecked( m_activeView->editPart()->isRichTextEnabled() );
-}
-
-void KopeteChatWindow::toggleRichTextAction( bool enable )
-{
-	m_activeView->editPart()->setRichTextEnabled( enable );
-}
-
 void KopeteChatWindow::updateActions()
 {
 	updateSpellCheckAction();
-	updateRichTextAction();
 	updateChatSendFileAction();
 }
 
@@ -635,32 +620,31 @@ void KopeteChatWindow::slotPaste()
 	m_activeView->paste();
 }
 
-
-void KopeteChatWindow::slotSetFont()
-{
-	m_activeView->setFont();
-}
-
 void KopeteChatWindow::slotResetFontAndColor()
 {
 	m_activeView->resetFontAndColor();
 }
 
-void KopeteChatWindow::slotSetFgColor()
-{
-	m_activeView->setFgColor();
-}
-
-void KopeteChatWindow::slotSetBgColor()
-{
-	m_activeView->setBgColor();
-}
-
 void KopeteChatWindow::setStatus(const QString &text)
 {
 	m_status_text->setText(text);
 }
 
+void KopeteChatWindow::testCanDecode(const QDragMoveEvent *event, bool &accept)
+{
+	if ( m_tabBar && chatViewList[static_cast<KTabBar*>(m_tabBar->childAt( event->pos()))->selectTab( event->pos() )]->isDragEventAccepted( event )) {
+		accept = true;
+	} else {
+		accept = false;
+	}
+}
+
+void KopeteChatWindow::receivedDropEvent( QWidget *w, QDropEvent *e )
+{
+	m_tabBar->setCurrentWidget( w );
+	activeView()->dropEvent( e );
+}
+
 void KopeteChatWindow::createTabBar()
 {
 	if( !m_tabBar )
@@ -669,8 +653,8 @@ void KopeteChatWindow::createTabBar()
 
 		m_tabBar = new KTabWidget( mainArea );
 		m_tabBar->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
-		m_tabBar->setHoverCloseButton(cg.readEntry( QLatin1String("HoverClose"), false ));
-		m_tabBar->setTabReorderingEnabled(true);
+		m_tabBar->setTabsClosable(cg.readEntry( QLatin1String("HoverClose"), false ));
+		m_tabBar->setMovable(true);
 		m_tabBar->setAutomaticResizeTabs(true);
 		connect( m_tabBar, SIGNAL( closeRequest( QWidget* )), this, SLOT( slotCloseChat( QWidget* ) ) );
 
@@ -687,6 +671,8 @@ void KopeteChatWindow::createTabBar()
 		for( ChatViewList::iterator it = chatViewList.begin(); it != chatViewList.end(); ++it )
 			addTab( *it );
 
+		connect ( m_tabBar, SIGNAL(testCanDecode(const QDragMoveEvent *, bool &)), this, SLOT(testCanDecode(const QDragMoveEvent *, bool &)) );
+		connect ( m_tabBar, SIGNAL(receivedDropEvent( QWidget *, QDropEvent * )), this, SLOT(receivedDropEvent( QWidget *, QDropEvent * )) );
 		connect ( m_tabBar, SIGNAL(currentChanged(QWidget *)), this, SLOT(setActiveView(QWidget *)) );
 		connect ( m_tabBar, SIGNAL(contextMenu(QWidget *, const QPoint & )), this, SLOT(slotTabContextMenu( QWidget *, const QPoint & )) );
 
@@ -737,11 +723,9 @@ void KopeteChatWindow::setPrimaryChatView( ChatView *view )
 {
 	//TODO figure out what else we have to save here besides the font
 	//reparent clears a lot of stuff out
-	QFont savedFont = view->font();
 	view->setParent( mainArea );
 	view->setWindowFlags( 0 );
 	view->move( QPoint() );
-	view->setFont( savedFont );
 	view->show();
 
 	mainLayout->addWidget( view );
@@ -796,6 +780,7 @@ void KopeteChatWindow::attachChatView( ChatView* newView )
 void KopeteChatWindow::checkDetachEnable()
 {
 	bool haveTabs = (chatViewList.count() > 1);
+	tabCloseAllOthers->setEnabled( haveTabs );
 	tabDetach->setEnabled( haveTabs );
 	tabLeft->setEnabled( haveTabs );
 	tabRight->setEnabled( haveTabs );
@@ -878,6 +863,21 @@ void KopeteChatWindow::slotDetachChat( QAction *action )
 	newWindow->attachChatView( detachedView );
 }
 
+void KopeteChatWindow::slotCloseAllOtherTabs()
+{
+	ChatView *detachedView;
+
+	if( m_popupView )
+		detachedView = m_popupView;
+	else
+		detachedView = m_activeView;
+
+	foreach(ChatView *view, chatViewList) {
+		if (view != detachedView)
+			view->closeView();
+	}
+}
+
 void KopeteChatWindow::slotPreviousTab()
 {
 	int curPage = m_tabBar->currentIndex();
@@ -946,7 +946,6 @@ void KopeteChatWindow::setActiveView( QWidget *widget )
 	if(m_activeView)
 	{
 		disconnect( m_activeView->editWidget(), SIGNAL(checkSpellingChanged(bool)), this, SLOT(enableSpellCheckAction(bool)) );
-		disconnect( m_activeView->editPart(), SIGNAL(richTextChanged()), this, SLOT(updateRichTextAction()) );
 		disconnect( m_activeView, SIGNAL(canSendChanged(bool)), this, SLOT(slotUpdateSendEnabled()) );
 		disconnect( m_activeView, SIGNAL(canAcceptFilesChanged()), this, SLOT(updateChatSendFileAction()) );
 		guiFactory()->removeClient(m_activeView->msgManager());
@@ -969,7 +968,6 @@ void KopeteChatWindow::setActiveView( QWidget *widget )
 		attachChatView( view );
 
 	connect( m_activeView->editWidget(), SIGNAL(checkSpellingChanged(bool)), this, SLOT(enableSpellCheckAction(bool)) );
-	connect( m_activeView->editPart(), SIGNAL(richTextChanged()), this, SLOT(updateRichTextAction()) );
 	connect( m_activeView, SIGNAL(canSendChanged(bool)), this, SLOT(slotUpdateSendEnabled()) );
 	connect( m_activeView, SIGNAL(canAcceptFilesChanged()), this, SLOT(updateChatSendFileAction()) );
 
@@ -1005,6 +1003,7 @@ void KopeteChatWindow::setActiveView( QWidget *widget )
 	updateActions();
 	slotUpdateSendEnabled();
 	m_activeView->loadChatSettings();
+	updateSendKeySequence();
 
 	emit chatSessionChanged(m_activeView->msgManager());
 }
@@ -1132,19 +1131,27 @@ void KopeteChatWindow::slotPreparePlacementMenu()
 
 	action = placementMenu->addAction( i18n("Bottom") );
 	action->setData( 1 );
+
+	action = placementMenu->addAction( i18n("Left") );
+	action->setData( 2 );
+
+	action = placementMenu->addAction( i18n("Right") );
+	action->setData( 3 );
 }
 
 void KopeteChatWindow::slotPlaceTabs( QAction *action )
 {
 	int placement = action->data().toInt();
+
 	if( m_tabBar )
 	{
-
-		if( placement == 0 )
-			m_tabBar->setTabPosition( QTabWidget::North );
-		else
-			m_tabBar->setTabPosition( QTabWidget::South );
-
+		switch( placement )
+		{
+		case 1 : m_tabBar->setTabPosition( QTabWidget::South ); break;
+		case 2 : m_tabBar->setTabPosition( QTabWidget::West  ); break;
+		case 3 : m_tabBar->setTabPosition( QTabWidget::East  ); break;
+		default: m_tabBar->setTabPosition( QTabWidget::North );
+		}
 		saveOptions();
 	}
 }
@@ -1320,15 +1327,14 @@ void KopeteChatWindow::updateChatTooltip( ChatView* cv )
 
 void KopeteChatWindow::updateChatLabel()
 {
-	const ChatView* cv = dynamic_cast<const ChatView*>( sender() );
-	if ( !cv || !m_tabBar )
+	ChatView* chat = dynamic_cast<ChatView*>( sender() );
+	if ( !chat || !m_tabBar )
 		return;
 
-	ChatView* chat  = const_cast<ChatView*>( cv );
 	if ( m_tabBar )
 	{
 		m_tabBar->setTabText( m_tabBar->indexOf( chat ), chat->caption() );
-		if ( m_tabBar->count() < 2 || m_tabBar->currentWidget() == static_cast<const QWidget *>(cv) )
+		if ( m_tabBar->count() < 2 || m_tabBar->currentWidget() == chat )
 			setCaption( chat->caption() );
 	}
 }
