@@ -28,7 +28,6 @@
 #include <kdialog.h>
 #include <kdeversion.h>
 #include <klocale.h>
-#include <kiconloader.h>
 #include <kiconeffect.h>
 #include <kicon.h>
 #include <kaction.h>
@@ -85,12 +84,14 @@ public:
 	Identity *identity;
 	Contact *myself;
 	QTimer suppressStatusTimer;
+	QTimer reconnectTimer;
 	bool suppressStatusNotification;
 	Kopete::BlackLister *blackList;
 	KConfigGroup *configGroup;
 	QString customIcon;
 	Kopete::OnlineStatus restoreStatus;
 	Kopete::StatusMessage restoreMessage;
+	QDateTime lastLoginTime;
 };
 
 Account::Account( Protocol *parent, const QString &accountId )
@@ -106,13 +107,16 @@ Account::Account( Protocol *parent, const QString &accountId )
 	d->restoreStatus = Kopete::OnlineStatus::Online;
 	d->restoreMessage = Kopete::StatusMessage();
 
+	d->reconnectTimer.setSingleShot( true );
+	QObject::connect( &d->reconnectTimer, SIGNAL(timeout()),
+	                  this, SLOT(reconnect()) );
+
 	QObject::connect( &d->suppressStatusTimer, SIGNAL( timeout() ),
 		this, SLOT( slotStopSuppression() ) );
 }
 
 Account::~Account()
 {
-	d->contacts.remove( d->myself->contactId() );
 	// Delete all registered child contacts first
 	foreach (Contact* c, d->contacts) QObject::disconnect(c, SIGNAL( contactDestroyed( Kopete::Contact * ) ), this, 0);
 	qDeleteAll(d->contacts);
@@ -128,6 +132,9 @@ Account::~Account()
 
 void Account::reconnect()
 {
+	if ( isConnected() )
+		return; // Already connected
+
 	kDebug( 14010 ) << "account " << d->id << " restoreStatus " << d->restoreStatus.status()
 	                << " restoreTitle " << d->restoreMessage.title()
 	                << " restoreMessage " << d->restoreMessage.message();
@@ -138,18 +145,28 @@ void Account::disconnected( DisconnectReason reason )
 {
 	kDebug( 14010 ) << reason;
 	//reconnect if needed
-	if(reason == BadPassword )
+	if ( reason == BadPassword )
 	{
-		QTimer::singleShot(0, this, SLOT(reconnect()));
+		d->reconnectTimer.start( 0 );
 	}
 	else if ( Kopete::BehaviorSettings::self()->reconnectOnDisconnect() == true && reason > Manual )
 	{
+		if ( d->reconnectTimer.isActive() )
+			return; // In case protocol calls disconnected more than one time on disconnect.
+
 		d->connectionTry++;
 		//use a timer to allow the plugins to clean up after return
-		if(d->connectionTry < 3)
-			QTimer::singleShot(10000, this, SLOT(reconnect())); // wait 10 seconds before reconnect
+		if ( d->connectionTry < 3 )
+			d->reconnectTimer.start( 10000 ); // wait 10 seconds before reconnect
+		else if ( d->connectionTry <= 10 )
+			d->reconnectTimer.start( ((2 * (d->connectionTry - 2)) - 1) * 60000 ); // wait 1,3,5...15 minutes => stops after 64 min
 	}
-	if(reason== OtherClient)
+	else
+	{
+		d->reconnectTimer.stop();
+	}
+
+	if ( reason == OtherClient )
 	{
 		Kopete::Utils::notifyConnectionLost(this, i18n("You have been disconnected"), i18n( "You have connected from another client or computer to the account '%1'" , d->id), i18n("Most proprietary Instant Messaging services do not allow you to connect from more than one location. Check that nobody is using your account without your permission. If you need a service that supports connection from various locations at the same time, use the Jabber protocol."));
 	}
@@ -214,6 +231,13 @@ QPixmap Account::accountIcon(const int size) const
 	return base;
 }
 
+QString Account::accountIconPath(const KIconLoader::Group size) const
+{
+	return KIconLoader::global()->iconPath(d->customIcon.isEmpty() ?
+	                                       d->protocol->pluginIcon() :
+					       d->customIcon, size);
+}
+
 KConfigGroup* Kopete::Account::configGroup() const
 {
 	return d->configGroup;
@@ -242,17 +266,21 @@ bool Account::excludeConnect() const
 	return d->excludeconnect;
 }
 
-void Account::registerContact( Contact *c )
+bool Account::registerContact( Contact *c )
 {
-	if ( d->contacts.value( c->contactId(), 0 ) )
+	Q_ASSERT ( c->metaContact() != Kopete::ContactList::self()->myself() );
+
+	if ( d->contacts.value( c->contactId() ) )
 	{
 		kWarning(14010) << "Contact already exists!!! accountId: " << c->account() << " contactId: " << c->contactId();
-		return;
+		return false;
 	}
 
 	d->contacts.insert( c->contactId(), c );
 	QObject::connect( c, SIGNAL( contactDestroyed( Kopete::Contact * ) ),
 		SLOT( contactDestroyed( Kopete::Contact * ) ) );
+
+	return true;
 }
 
 void Account::contactDestroyed( Contact *c )
@@ -270,17 +298,25 @@ const QHash<QString, Contact*>& Account::contacts()
 Kopete::MetaContact* Account::addContact( const QString &contactId, const QString &displayName , Group *group, AddMode mode  )
 {
 
-	if ( contactId == d->myself->contactId() )
+	if ( !protocol()->canAddMyself() && contactId == d->myself->contactId() )
 	{
-		KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error,
-			i18n("You are not allowed to add yourself to the contact list. The addition of \"%1\" to account \"%2\" will not take place.", contactId, accountId()), i18n("Error Creating Contact")
-		);
+		if ( isConnected() && d->lastLoginTime.secsTo(QDateTime::currentDateTime()) > 30 )
+		{
+			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error,
+			                               i18n("You are not allowed to add yourself to the contact list. The addition of \"%1\" to account \"%2\" will not take place.", contactId, accountId()), i18n("Error Creating Contact")
+			                             );
+		}
+		else
+		{
+			kWarning(14010) << "You are not allowed to add yourself to the contact list. The addition of" << contactId
+			                << "to account" << accountId() << "will not take place.";
+		}
 		return false;
 	}
 
-	bool isTemporary = mode == Temporary;
+	bool isTemporary = (mode == Temporary);
 
-	Contact *c = d->contacts[ contactId ];
+	Contact *c = d->contacts.value( contactId );
 
 	if(!group)
 		group=Group::topLevel();
@@ -336,16 +372,25 @@ Kopete::MetaContact* Account::addContact( const QString &contactId, const QStrin
 
 bool Account::addContact(const QString &contactId , MetaContact *parent, AddMode mode )
 {
-	if ( contactId == myself()->contactId() )
+	if ( !protocol()->canAddMyself() && contactId == myself()->contactId() )
 	{
-	    	KMessageBox::error( Kopete::UI::Global::mainWidget(),
-			i18n("You are not allowed to add yourself to the contact list. The addition of \"%1\" to account \"%2\" will not take place.", contactId, accountId()), i18n("Error Creating Contact")
-		);
+		if ( isConnected() && d->lastLoginTime.secsTo(QDateTime::currentDateTime()) > 30 )
+		{
+			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error,
+			                               i18n("You are not allowed to add yourself to the contact list. The addition of \"%1\" to account \"%2\" will not take place.", contactId, accountId()), i18n("Error Creating Contact")
+			                             );
+		}
+		else
+		{
+			kWarning(14010) << "You are not allowed to add yourself to the contact list. The addition of" << contactId
+			                << "to account" << accountId() << "will not take place.";
+		}
+		
 		return 0L;
 	}
 
-	bool isTemporary= parent->isTemporary();
-	Contact *c = d->contacts[ contactId ];
+	const bool isTemporary= parent->isTemporary();
+	Contact *c = d->contacts.value( contactId );
 	if ( c && c->metaContact() )
 	{
 		if ( c->metaContact()->isTemporary() && !isTemporary )
@@ -365,7 +410,7 @@ bool Account::addContact(const QString &contactId , MetaContact *parent, AddMode
 		return false; //(the contact is not in the correct metacontact, so false)
 	}
 
-	bool success = createContact(contactId, parent);
+	const bool success = createContact(contactId, parent);
 
 	if ( success && mode == ChangeKABC )
 	{
@@ -415,8 +460,11 @@ bool Account::isConnected() const
 
 bool Account::isAway() const
 {
-	return d->myself && ( d->myself->onlineStatus().status() == Kopete::OnlineStatus::Away );
+	//We might want to change the method name here.
+	return d->myself && ( d->myself->onlineStatus().status() == Kopete::OnlineStatus::Away ||
+					d->myself->onlineStatus().status() == Kopete::OnlineStatus::Busy );
 }
+
 Identity * Account::identity() const
 {
 	return d->identity;
@@ -447,36 +495,27 @@ Contact * Account::myself() const
 
 void Account::setMyself( Contact *myself )
 {
-	//FIXME  does it make sens to change the myself contact to another ?   - Olivier 2005-11-21
-
-	bool wasConnected = isConnected();
-
-	if ( d->myself )
-	{
-		QObject::disconnect( d->myself, SIGNAL( onlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
-			this, SLOT( slotOnlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ) );
-		QObject::disconnect( d->myself, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ),
-			this, SLOT( slotContactPropertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ) );
-	}
+	Q_ASSERT( !d->myself );
 
 	d->myself = myself;
 
-//	d->contacts.remove( myself->contactId() );
-
 	QObject::connect( d->myself, SIGNAL( onlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
 		this, SLOT( slotOnlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ) );
 	QObject::connect( d->myself, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ),
 		this, SLOT( slotContactPropertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ) );
 
-	if ( isConnected() != wasConnected )
+	if ( isConnected() )
 		emit isConnectedChanged();
 }
 
 void Account::slotOnlineStatusChanged( Contact * /* contact */,
 	const OnlineStatus &newStatus, const OnlineStatus &oldStatus )
 {
-	bool wasOffline = !oldStatus.isDefinitelyOnline();
-	bool isOffline  = !newStatus.isDefinitelyOnline();
+	const bool wasOffline = !oldStatus.isDefinitelyOnline();
+	const bool isOffline  = !newStatus.isDefinitelyOnline();
+
+	if (wasOffline && !isOffline)
+		d->lastLoginTime = QDateTime::currentDateTime();
 
 	if ( wasOffline || newStatus.status() == OnlineStatus::Offline )
 	{
@@ -517,7 +556,7 @@ void Account::setAllContactsStatus( const Kopete::OnlineStatus &status )
 		it.next();
 
 		Contact *c = it.value();
-		if ( c && c != d->myself )
+		if ( c )
 			c->setOnlineStatus( status );
 	}
 }
@@ -578,8 +617,10 @@ void Account::editAccount(QWidget *parent)
 
 	KopeteEditAccountWidget *m_accountWidget = protocol()->createEditAccountWidget( this, editDialog );
 	if ( !m_accountWidget )
+	{
+		delete editDialog;
 		return;
-
+	}
 	// FIXME: Why the #### is EditAccountWidget not a QWidget?!? This sideways casting
 	//        is braindead and error-prone. Looking at MSN the only reason I can see is
 	//        because it allows direct subclassing of designer widgets. But what is
@@ -588,8 +629,10 @@ void Account::editAccount(QWidget *parent)
 	//        class should at least be renamed to EditAccountIface instead - Martijn
 	QWidget *w = dynamic_cast<QWidget *>( m_accountWidget );
 	if ( !w )
+	{
+		delete editDialog;
 		return;
-
+	}
 	editDialog->setMainWidget( w );
 	if ( editDialog->exec() == QDialog::Accepted )
 	{
