@@ -23,7 +23,9 @@
 #include "kopetechatsession.h"
 
 #include <qapplication.h>
+#include <qfile.h>
 #include <qregexp.h>
+#include <qpointer.h>
 
 #include <kdebug.h>
 #include <kdeversion.h>
@@ -31,6 +33,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <knotification.h>
+#include <kstandarddirs.h>
 
 #include "kopeteaccount.h"
 #include "kopetebehaviorsettings.h"
@@ -38,18 +41,21 @@
 #include "kopetechatsessionmanager.h"
 #include "kopetemessagehandlerchain.h"
 #include "kopetemetacontact.h"
+#include "kopetegroup.h"
 #include "kopeteuiglobal.h"
 #include "kopeteglobal.h"
 #include "kopeteview.h"
 #include "kopetecontact.h"
 #include "kopetepluginmanager.h"
+#include "kopeteprotocol.h"
+#include "kopetepicture.h"
 
 const int CHAIN_COUNT = 3;
 
 class KMMPrivate
 {
 public:
-	Kopete::ContactPtrList mContactList;
+	Kopete::ContactPtrList contacts;
 	const Kopete::Contact *mUser;
 	QMap<const Kopete::Contact *, Kopete::OnlineStatus> contactStatus;
 	Kopete::Protocol *mProtocol;
@@ -59,10 +65,12 @@ public:
 	bool customDisplayName;
 	QDateTime awayTime;
 	QString displayName;
+	QString lastUrl;
 	KopeteView *view;
 	bool mayInvite;
 	Kopete::MessageHandlerChain::Ptr chains[CHAIN_COUNT];
 	Kopete::ChatSession::Form form;
+	bool warnGroupChat;
 };
 
 Kopete::ChatSession::ChatSession( const Kopete::Contact *user,
@@ -79,6 +87,10 @@ Kopete::ChatSession::ChatSession( const Kopete::Contact *user,
 	d->customDisplayName = false;
 	d->mayInvite = false;
 	d->form = form;
+	d->warnGroupChat = true;
+	if ( !others.isEmpty() ) {
+		d->lastUrl = initLastUrl( others.first() );
+	}
 
 	for ( int i = 0; others.size() != i; ++i )
 		addContact( others[i], true );
@@ -98,7 +110,7 @@ Kopete::ChatSession::ChatSession( const Kopete::Contact *user,
 
 Kopete::ChatSession::~ChatSession()
 {
-	//for ( Kopete::Contact *c = d->mContactList.first(); c; c = d->mContactList.next() )
+	//for ( Kopete::Contact *c = d->contacts.first(); c; c = d->contacts.next() )
 	//	c->setConversations( c->conversations() - 1 );
 
 	if ( !d )
@@ -132,6 +144,16 @@ const Kopete::OnlineStatus Kopete::ChatSession::contactOnlineStatus( const Kopet
 	return contact->onlineStatus();
 }
 
+void Kopete::ChatSession::setLastUrl( const QString &verylastUrl )
+{
+	  d->lastUrl = verylastUrl;
+}
+
+const QString Kopete::ChatSession::lastUrl()
+{
+	return d->lastUrl;
+}
+
 const QString Kopete::ChatSession::displayName()
 {
 	if ( d->displayName.isNull() )
@@ -156,13 +178,13 @@ void Kopete::ChatSession::slotUpdateDisplayName()
 
 
 	//If there is no member yet, don't try to update the display name
-	if ( d->mContactList.isEmpty() )
+	if ( d->contacts.isEmpty() )
 		return;
 
 	d->displayName.clear();
-	for(int i = 0; i != d->mContactList.size(); i++ )
+	for(int i = 0; i != d->contacts.size(); i++ )
 	{
-		Kopete::Contact * c = d->mContactList[i];
+		Kopete::Contact * c = d->contacts[i];
 		if(! d->displayName.isNull() )
 			d->displayName.append( QString::fromLatin1( ", " ) ) ;
 
@@ -176,9 +198,9 @@ void Kopete::ChatSession::slotUpdateDisplayName()
 	}
 
 	//If we have only 1 contact, add the status of him
-	if ( d->mContactList.count() == 1 )
+	if ( d->contacts.count() == 1 )
 	{
-		d->displayName.append( QString::fromLatin1( " (%1)" ).arg( d->mContactList.first()->onlineStatus().description() ) );
+		d->displayName.append( QString::fromLatin1( " (%1)" ).arg( d->contacts.first()->onlineStatus().description() ) );
 	}
 
 	emit displayNameChanged();
@@ -186,7 +208,7 @@ void Kopete::ChatSession::slotUpdateDisplayName()
 
 const Kopete::ContactPtrList& Kopete::ChatSession::members() const
 {
-	return d->mContactList;
+	return d->contacts;
 }
 
 const Kopete::Contact* Kopete::ChatSession::myself() const
@@ -206,7 +228,7 @@ Kopete::Protocol* Kopete::ChatSession::protocol() const
 // FIXME: remove this and the friend decl in KMM
 class Kopete::TemporaryKMMCallbackAppendMessageHandler : public Kopete::MessageHandler
 {
-	Kopete::ChatSession *manager;
+	QPointer<Kopete::ChatSession> manager;
 public:
 	TemporaryKMMCallbackAppendMessageHandler( Kopete::ChatSession *manager )
 	: manager(manager)
@@ -215,7 +237,10 @@ public:
 	void handleMessage( Kopete::MessageEvent *event )
 	{
 		Kopete::Message message = event->message();
-		emit manager->messageAppended( message, manager );
+
+		if ( manager )
+			emit manager->messageAppended( message, manager );
+
 		delete event;
 	}
 };
@@ -286,10 +311,10 @@ void Kopete::ChatSession::appendMessage( Kopete::Message &msg )
 
 	if ( msg.direction() == Kopete::Message::Inbound )
 	{
-		QString nick = myself()->nickName();
+		const QString nick = myself()->nickName();
 		if ( Kopete::BehaviorSettings::self()->highlightEnabled() && !nick.isEmpty() )
 		{
-			QString nickNameRegExp = QString::fromLatin1( "(^|[\\W])(%1)([\\W]|$)" ).arg( QRegExp::escape( nick ) );
+			const QString nickNameRegExp = QString::fromLatin1( "(^|[\\W])(%1)([\\W]|$)" ).arg( QRegExp::escape( nick ) );
 			if ( msg.plainBody().contains( QRegExp( nickNameRegExp, Qt::CaseInsensitive ) ) )
 			{
 				msg.setImportance( Kopete::Message::Highlight );
@@ -307,9 +332,101 @@ void Kopete::ChatSession::appendMessage( Kopete::Message &msg )
 		chainDirection = Kopete::Message::Inbound;
 
 	chainForDirection( chainDirection )->processMessage( msg );
+
+	//looking for urls in the message
+	urlSearch( msg );
 //	emit messageAppended( msg, this );
 }
 
+void Kopete::ChatSession::urlSearch( const Kopete::Message &msg )
+{
+	//if there are any urls in the message
+	QStringList lasturls = findUrls(msg);
+	if ( !lasturls.empty() ) {
+		//set lasturl for message's chatsession
+		msg.manager()->setLastUrl( lasturls.last() );
+		//saving new url(s) found in message //file named contactId.lasturls.txt in proper folder
+		QString urlfilename = Kopete::ChatSession::getUrlsFileName( msg.manager()->members().first() );
+		QFile file( urlfilename );
+		file.open( QIODevice::Append );
+		QTextStream stream( &file );
+
+		for (int i = 0; i < lasturls.size(); ++i)
+			stream << lasturls.at(i) << "\n";
+		file.close();
+	}
+}
+
+QStringList Kopete::ChatSession::findUrls(const Kopete::Message &msg )
+{
+	Kopete::Message message = msg;
+	//we check the message for every pattern
+	QString tempstr = message.plainBody();
+	QStringList regexppatterns = message.regexpPatterns();
+	QRegExp linkregexp;
+	QMap<int,QString> mapUrl;
+
+	for (int i = 0; i < regexppatterns.size(); ++i) {
+	  linkregexp.setPattern(regexppatterns[i]);
+	  int pos = 0;
+	  while ((pos = linkregexp.indexIn(tempstr, pos)) != -1) {
+	    mapUrl.insert(pos,linkregexp.cap(0));
+	    pos += linkregexp.matchedLength(); }
+	}
+	//we use QMap to sort links as they are in the message (if there are many links in one message)
+	//lasturllist[0] - is the earliest
+	QStringList lasturllist;
+	QMapIterator< int, QString > i(mapUrl);
+	while (i.hasNext()) { i.next(); lasturllist << i.value(); }
+	lasturllist.replaceInStrings(" ", "");
+	//add "http://" to link if needed to open it with a browser
+	lasturllist.replaceInStrings(QRegExp( regexppatterns[1] ), QLatin1String("\\1http://\\2\\3" ));
+
+	return lasturllist;
+}
+
+QString Kopete::ChatSession::initLastUrl( const Kopete::Contact* c )
+{
+	QString urlfilename = getUrlsFileName(c);
+	if ( !urlfilename.isEmpty() )
+	{
+		QFile file( urlfilename );
+		QString lastUrl;
+		if ( file.exists() ) {
+			if ( file.open(QIODevice::ReadOnly) ) {
+				QTextStream stream( &file );
+				while ( !stream.atEnd() )
+					lastUrl = stream.readLine();
+				file.close(); }
+			else {
+				kDebug(14310) << "cant open lasturls file for " << c->contactId();
+			}
+		}
+		if ( !lastUrl.isEmpty() )
+			return lastUrl;
+		else return "";
+	}
+	else
+	{
+		kDebug(14310) << "cant find lasturls file for " << c->contactId();
+		return "";
+	}
+}
+
+QString Kopete::ChatSession::getUrlsFileName(const Kopete::Contact* c)
+{
+	QString name = c->protocol()->pluginId().replace( QRegExp( QString::fromLatin1( "[./~?*]" ) ), QString::fromLatin1( "-" ) ) +
+		QString::fromLatin1( "/" ) +
+		c->account()->accountId().replace( QRegExp( QString::fromLatin1( "[./~?*]" ) ), QString::fromLatin1( "-" ) ) +
+		QString::fromLatin1( "/" ) +
+	c->contactId().replace( QRegExp( QString::fromLatin1( "[./~?*]" ) ), QString::fromLatin1( "-" ) ) +
+		QString::fromLatin1( ".lasturls" );
+
+	QString filename = KStandardDirs::locateLocal( "data", QString::fromLatin1( "kopete/urls/" ) + name + QString::fromLatin1( ".txt" ) ) ;
+
+	return filename;
+}
+
 void Kopete::ChatSession::addContact( const Kopete::Contact *c, const Kopete::OnlineStatus &initialStatus, bool suppress )
 {
 	if( !d->contactStatus.contains(c) )
@@ -320,21 +437,21 @@ void Kopete::ChatSession::addContact( const Kopete::Contact *c, const Kopete::On
 void Kopete::ChatSession::addContact( const Kopete::Contact *c, bool suppress )
 {
 	//kDebug( 14010 ) ;
-	if ( d->mContactList.contains( (Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)c ) )
+	if ( d->contacts.contains( (Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)(Kopete::Contact*)c ) )
 	{
 		kDebug( 14010 ) << "Contact already exists";
-		emit contactAdded( c, suppress );
+//		emit contactAdded( c, suppress );
 	}
 	else
 	{
-		if ( d->mContactList.count() == 1 && d->isEmpty )
+		if ( d->contacts.count() == 1 && d->isEmpty )
 		{
 			kDebug( 14010 ) << " FUCKER ZONE ";
 			/* We have only 1 contact before, so the status of the
 			   message manager was given from that contact status */
-			Kopete::Contact *old = d->mContactList.first();
-			d->mContactList.removeAll( old );
-			d->mContactList.append( (Kopete::Contact*)c );
+			Kopete::Contact *old = d->contacts.first();
+			d->contacts.removeAll( old );
+			d->contacts.append( (Kopete::Contact*)c );
 
 			disconnect( old, SIGNAL( onlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
 			this, SLOT( slotOnlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus &) ) );
@@ -346,12 +463,15 @@ void Kopete::ChatSession::addContact( const Kopete::Contact *c, bool suppress )
 			}
 			else
 				disconnect( old, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT( slotUpdateDisplayName() ) );
+
+			disconnect( old, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT(slotContactPropertyChanged(Kopete::PropertyContainer*,QString,QVariant,QVariant) ) );
+
 			emit contactAdded( c, suppress );
 			emit contactRemoved( old, QString() );
 		}
 		else
 		{
-			d->mContactList.append( (Kopete::Contact*)c );
+			d->contacts.append( (Kopete::Contact*)c );
 			emit contactAdded( c, suppress );
 		}
 
@@ -366,6 +486,7 @@ void Kopete::ChatSession::addContact( const Kopete::Contact *c, bool suppress )
 		else
 			connect( c, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT( slotUpdateDisplayName() ) );
 		connect( c, SIGNAL( contactDestroyed( Kopete::Contact * ) ), this, SLOT( slotContactDestroyed( Kopete::Contact * ) ) );
+		connect( c, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT(slotContactPropertyChanged(Kopete::PropertyContainer*,QString,QVariant,QVariant) ) );
 
 		slotUpdateDisplayName();
 	}
@@ -375,17 +496,17 @@ void Kopete::ChatSession::addContact( const Kopete::Contact *c, bool suppress )
 void Kopete::ChatSession::removeContact( const Kopete::Contact *c, const QString& reason, Qt::TextFormat format, bool suppressNotification )
 {
 	kDebug( 14010 ) ;
-	if ( !c || !d->mContactList.contains( (Kopete::Contact*)c ) )
+	if ( !c || !d->contacts.contains( (Kopete::Contact*)c ) )
 		return;
 
-	if ( d->mContactList.count() == 1 )
+	if ( d->contacts.count() == 1 )
 	{
 		kDebug( 14010 ) << "Contact not removed. Keep always one contact";
 		d->isEmpty = true;
 	}
 	else
 	{
-		d->mContactList.removeAll( (Kopete::Contact*)c );
+		d->contacts.removeAll( (Kopete::Contact*)c );
 
 		disconnect( c, SIGNAL( onlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
 			this, SLOT( slotOnlineStatusChanged( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus &) ) );
@@ -399,6 +520,8 @@ void Kopete::ChatSession::removeContact( const Kopete::Contact *c, const QString
 			disconnect( c, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT( slotUpdateDisplayName() ) );
 		disconnect( c, SIGNAL( contactDestroyed( Kopete::Contact * ) ), this, SLOT( slotContactDestroyed( Kopete::Contact * ) ) );
 
+		disconnect( c, SIGNAL( propertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & ) ), this, SLOT(slotContactPropertyChanged(Kopete::PropertyContainer*,QString,QVariant,QVariant) ) );
+
 		slotUpdateDisplayName();
 	}
 
@@ -409,20 +532,49 @@ void Kopete::ChatSession::removeContact( const Kopete::Contact *c, const QString
 
 void Kopete::ChatSession::receivedTypingMsg( const Kopete::Contact *c, bool t )
 {
+	if ( t )
+	{
+		QWidget *viewWidget = 0L;
+		bool isActiveWindow = false;
+
+		if ( !account()->isAway() || Kopete::BehaviorSettings::self()->enableEventsWhileAway() )
+		{
+			viewWidget = dynamic_cast<QWidget*>(view(false));
+			isActiveWindow = view(false) && ( viewWidget && viewWidget->isActiveWindow() ) ;
+		}
+
+		if ( ! isActiveWindow )
+		{
+			KNotification * notification = new KNotification ( "user_is_typing_message", Kopete::UI::Global::mainWidget() );
+			notification->setText( i18n("User <i>%1</i> is typing a message", c->nickName()) );
+			notification->setPixmap( QPixmap::fromImage( c->metaContact()->picture().image() ) );
+			notification->setActions( QStringList( i18nc("@action", "Chat") ) );
+
+			notification->addContext( qMakePair( QString::fromLatin1("contact"), c->metaContact()->metaContactId().toString() ) );
+			foreach( Kopete::Group *g , c->metaContact()->groups() )
+			{
+				notification->addContext( qMakePair( QString::fromLatin1("group") , QString::number( g->groupId() ) ) );
+			}
+
+			connect( notification, SIGNAL(activated(unsigned int)) , c, SLOT(execute()) );
+			notification->sendEvent();
+		}
+	}
+
 	emit remoteTyping( c, t );
 }
 
 void Kopete::ChatSession::receivedTypingMsg( const QString &contactId, bool t )
 {
 	int i;
-	
+
 	// FIXME: this needs better design. We can't iterate through List to find out who got what ID
 	// hash will be better for that, right ?
-	for ( i=0; i != d->mContactList.size(); i++ )
+	for ( i=0; i != d->contacts.size(); i++ )
 	{
-		if ( (d->mContactList[i])->contactId() == contactId )
+		if ( (d->contacts[i])->contactId() == contactId )
 		{
-			receivedTypingMsg( d->mContactList[i], t );
+			receivedTypingMsg( d->contacts[i], t );
 			return;
 		}
 	}
@@ -497,21 +649,22 @@ Kopete::Account *Kopete::ChatSession::account() const
 
 void Kopete::ChatSession::slotContactDestroyed( Kopete::Contact *contact )
 {
-	if( !contact || !d->mContactList.contains( contact ) )
+	if( !contact || !d->contacts.contains( contact ) )
 		return;
 
 	//This is a workaround to prevent crash if the contact get deleted.
 	// in the best case, we should ask the protocol to recreate a temporary contact.
 	// (remember: the contact may be deleted when the users removes it from the contact list, or when closing kopete )
-	d->mContactList.removeAll( contact );
+	d->contacts.removeAll( contact );
 	emit contactRemoved( contact, QString() );
 
-	if ( d->mContactList.isEmpty() )
+	if ( d->contacts.isEmpty() )
 		deleteLater();
 }
 
 void Kopete::ChatSession::slotMyselfDestroyed( Kopete::Contact *contact )
 {
+	Q_UNUSED(contact);
 	d->mUser = 0;
 	deleteLater();
 }
@@ -542,6 +695,23 @@ Kopete::ChatSession::Form Kopete::ChatSession::form() const
 {
 	return d->form;
 }
+bool Kopete::ChatSession::warnGroupChat() const
+{
+	return d->warnGroupChat;
+}
+
+void Kopete::ChatSession::setWarnGroupChat( bool b )
+{
+	d->warnGroupChat=b;
+}
+
+void Kopete::ChatSession::slotContactPropertyChanged( Kopete::PropertyContainer * contact, const QString &key, const QVariant &oldValue, const QVariant &newValue ){
+	Kopete::Contact *c = (Kopete::Contact*) contact;
+	if (key == Kopete::Global::Properties::self()->nickName().key()){
+		const QString nick = oldValue.toString();
+		emit nickNameChanged(c,nick);
+	}
+}
 
 #include "kopetechatsession.moc"
 
