@@ -17,15 +17,17 @@
 */
 
 #include "historydialog.h"
-#include "historylogger.h"
-#include "ui_historyviewer.h"
-#include "kopetemetacontact.h"
-#include "kopeteprotocol.h"
-#include "kopeteaccount.h"
-#include "kopetecontact.h"
-#include "kopetecontactlist.h"
-#include "kopeteappearancesettings.h"
 
+#include <QtCore/QDir>
+#include <QtCore/QTextStream>
+#include <QtGui/QClipboard>
+#include <QtGui/QTextDocument>
+
+#include <kdebug.h>
+#include <krun.h>
+#include <kmenu.h>
+#include <kaction.h>
+#include <kstandardaction.h>
 #include <dom/dom_doc.h>
 #include <dom/dom_element.h>
 #include <dom/html_document.h>
@@ -33,15 +35,16 @@
 #include <khtml_part.h>
 #include <khtmlview.h>
 
-#include <QDir>
-#include <QClipboard>
-#include <QTextOStream>
+#include "kopetemetacontact.h"
+#include "kopeteprotocol.h"
+#include "kopeteaccount.h"
+#include "kopetecontact.h"
+#include "kopetecontactlist.h"
+#include "kopeteappearancesettings.h"
 
-#include <kdebug.h>
-#include <krun.h>
-#include <kmenu.h>
-#include <kaction.h>
-#include <kstandardaction.h>
+#include "historylogger.h"
+#include "historyimport.h"
+#include "ui_historyviewer.h"
 
 class KListViewDateItem : public QTreeWidgetItem
 {
@@ -77,7 +80,8 @@ bool KListViewDateItem::operator<( const QTreeWidgetItem& other ) const
 
 
 HistoryDialog::HistoryDialog(Kopete::MetaContact *mc, QWidget* parent)
- : KDialog(parent)
+ : KDialog(parent),
+   mSearching(false)
 {
 	setAttribute (Qt::WA_DeleteOnClose, true);
 	setCaption( i18n("History for %1", mc->displayName()) );
@@ -88,8 +92,6 @@ HistoryDialog::HistoryDialog(Kopete::MetaContact *mc, QWidget* parent)
 
 	kDebug(14310) << "called.";
 
-	// Class member initializations
-	mSearch = 0L;
 
 	// FIXME: Allow to show this dialog for only one contact
 	mMetaContact = mc;
@@ -157,6 +159,7 @@ HistoryDialog::HistoryDialog(Kopete::MetaContact *mc, QWidget* parent)
 	connect(mMainWidget->searchLine, SIGNAL(textChanged(const QString&)), this, SLOT(slotSearchTextChanged(const QString&)));
 	connect(mMainWidget->contactComboBox, SIGNAL(activated(int)), this, SLOT(slotContactChanged(int)));
 	connect(mMainWidget->messageFilterBox, SIGNAL(activated(int)), this, SLOT(slotFilterChanged(int )));
+	connect(mMainWidget->importHistory, SIGNAL(clicked()), this, SLOT(slotImportHistory()));
 	connect(mHtmlPart, SIGNAL(popupMenu(const QString &, const QPoint &)), this, SLOT(slotRightClick(const QString &, const QPoint &)));
 
 	//initActions
@@ -179,8 +182,9 @@ HistoryDialog::HistoryDialog(Kopete::MetaContact *mc, QWidget* parent)
 
 HistoryDialog::~HistoryDialog()
 {
+	// end the search function, if it's still running
+	mSearching = false;
 	delete mMainWidget;
-	delete mSearch;
 }
 
 void HistoryDialog::init()
@@ -333,6 +337,8 @@ void HistoryDialog::setMessages(QList<Kopete::Message> msgs)
 	newNode.setInnerHTML(resultHTML);
 	mHtmlPart->htmlDocument().body().appendChild(newNode);
 
+	const QString searchForEscaped = Qt::escape(mMainWidget->searchLine->text());
+	
 	// Populating HTML Part with messages
 	foreach(const Kopete::Message& msg, msgs)
 	{
@@ -353,11 +359,9 @@ void HistoryDialog::setMessages(QList<Kopete::Message> msgs)
 
 			QString body = msg.parsedBody();
 
-			if (!mMainWidget->searchLine->text().isEmpty())
-			// If there is a search, then we hightlight the keywords
-			{
-				body = body.replace(mMainWidget->searchLine->text(), "<span style=\"background-color:yellow\">" + mMainWidget->searchLine->text() + "</span>", Qt::CaseInsensitive);
-			}
+			// If there is a search, then we highlight the keywords
+			if (!searchForEscaped.isEmpty() && body.contains(searchForEscaped, Qt::CaseInsensitive))
+				body = highlight( body, searchForEscaped );
 
 			QString name;
 			if ( msg.from()->metaContact() && msg.from()->metaContact() != Kopete::ContactList::self()->myself() )
@@ -428,31 +432,31 @@ void HistoryDialog::treeWidgetHideElements(bool s)
 	}
 }
 
-// Search initialization
+/*
+* How does the search work
+* ------------------------
+* We do the search respecting the current metacontact filter item. To do this, we iterate over the
+* elements in the KListView (KListViewDateItems) and, for each one, we iterate over its subcontacts,
+* manually searching the log files of each one. To avoid searching files twice, the months that have
+* been searched already are stored in monthsSearched. The matches are placed in the matches QMap.
+* Finally, the current date item is checked in the matches QMap, and if it is present, it is shown.
+*
+* Keyword highlighting is done in setMessages() : if the search field isn't empty, we highlight the
+* search keyword.
+*
+* The search is _not_ case sensitive
+*/
 void HistoryDialog::slotSearch()
 {
-	/*
-	* How does the search work
-	* ------------------------
-	* We do the search respecting the current metacontact filter item. So to do this, we iterate (searchFirstStep()) over
-	* the elements in the K3ListView (KListViewDateItems) and, for each one, we iterate over its subcontacts, retrieving the log
-	* files of each one, log files in which we do a fulltext search. If we match the keyword, we use dateSearchMap to mark which days
-	* in this file contain the keyword.
-	*
-	* Then, we only show the items for which there is a correct dateSearchMap
-	*
-	* Keyword highlighting is done in setMessages() : if the search field isn't empty, we highlight the search keyword.
-	*
-	* The search is _not_ case sensitive
-	* TODO: Speed up search ! Solutions : optimisations ?
-	*/
+	QRegExp rx("^[\\s]*(\\d+)[\\s]+\\d+:\\d+:\\d+[\\s]*$");
+	QMap<QDate, QList<Kopete::MetaContact*> > monthsSearched;
+	QMap<QDate, QList<Kopete::MetaContact*> > matches;
 
-	if (mSearch)
+	// cancel button pressed
+	if (mSearching)
 	{
-		mMainWidget->searchButton->setText(i18n("&Search"));
-		delete mSearch;
-		mSearch = 0L;
-		doneProgressBar();
+		treeWidgetHideElements(false);
+		searchFinished();
 		return;
 	}
 
@@ -460,83 +464,102 @@ void HistoryDialog::slotSearch()
 
 	treeWidgetHideElements(true);
 
-	mSearch = new Search();
-	mSearch->itemIndex = 0;
-	mSearch->foundPrevious = false;
-
-	initProgressBar(i18n("Searching..."), mMainWidget->dateTreeWidget->topLevelItemCount() );
+	initProgressBar(i18n("Searching..."), mMainWidget->dateTreeWidget->topLevelItemCount());
 	mMainWidget->searchButton->setText(i18n("&Cancel"));
+	mSearching = true;
 
-	searchFirstStep();
-}
+	QDomDocument doc;
 
-void HistoryDialog::searchFirstStep()
-{
-	QRegExp rx("^ <msg.*time=\"(\\d+) \\d+:\\d+:\\d+\" >");
+	QString searchFor = mMainWidget->searchLine->text();
+	// We are sarching in xml file so we need escaped search text too.
+	QString searchForEscaped = escapeXMLText(searchFor);
 
-	if (!mSearch)
+	const QString StartMsgTag("<msg");
+	const QString EndMsgTag("</msg");
+
+	// iterate over items in the date list widget
+	for(int i = 0; i < mMainWidget->dateTreeWidget->topLevelItemCount(); ++i)
 	{
-		return;
-	}
+		qApp->processEvents();
+		if (!mSearching) return;
 
-	KListViewDateItem *item = static_cast<KListViewDateItem*>(mMainWidget->dateTreeWidget->topLevelItem(mSearch->itemIndex));
-	if (item && !mSearch->dateSearchMap[item->date()].contains(item->metaContact()))
-	{
-		if (mMainWidget->contactComboBox->currentIndex() == 0
-		    || mMetaContactList.at(mMainWidget->contactComboBox->currentIndex()-1) == item->metaContact())
+		KListViewDateItem *curItem = static_cast<KListViewDateItem*>(mMainWidget->dateTreeWidget->topLevelItem(i));
+		QDate month(curItem->date().year(),curItem->date().month(),1);
+		// if we haven't searched the relevant history logs, search them now
+		if (!monthsSearched[month].contains(curItem->metaContact()))
 		{
-			HistoryLogger hlog(item->metaContact());
-
-			QList<Kopete::Contact*> contacts=item->metaContact()->contacts();
-
+			monthsSearched[month].push_back(curItem->metaContact());
+			QList<Kopete::Contact*> contacts = curItem->metaContact()->contacts();
 			foreach(Kopete::Contact* contact, contacts)
 			{
-				mSearch->datePrevious = item->date();
-
-				QString fullText;
-
-				QFile file(hlog.getFileName(contact, item->date()));
+				// get filename and open file
+				QString filename(HistoryLogger::getFileName(contact, curItem->date()));
+				if (!QFile::exists(filename)) continue;
+				QFile file(filename);
 				file.open(QIODevice::ReadOnly);
 				if (!file.isOpen())
 				{
+					kWarning(14310) << k_funcinfo << "Error opening " <<
+							file.fileName() << ": " << file.errorString() << endl;
 					continue;
 				}
+
 				QTextStream stream(&file);
 				QString textLine;
-				while(!(textLine = stream.readLine()).isNull())
+    			QString msgItem;
+				while(!stream.atEnd())
 				{
-					if (textLine.contains(mMainWidget->searchLine->text(), Qt::CaseInsensitive))
-					{
-						rx.indexIn(textLine);
-						mSearch->dateSearchMap[QDate(item->date().year(),item->date().month(),rx.cap(1).toInt())].push_back(item->metaContact());
-					}
-				}
+					textLine = stream.readLine();
+					if (!textLine.contains(StartMsgTag))
+						continue;
 
+					msgItem = textLine;
+					// Get whole message
+					while(!stream.atEnd() && !textLine.contains(EndMsgTag))
+						msgItem += textLine = stream.readLine();
+
+					if (msgItem.contains(searchForEscaped, Qt::CaseInsensitive))
+					{
+						// Load message
+						if (doc.setContent(msgItem))
+						{
+							// Check if only message body matches
+							if (doc.documentElement().text().contains(searchFor, Qt::CaseInsensitive))
+							{
+								if(rx.indexIn(doc.documentElement().attribute("time")) != -1)
+								{
+									QDate date(curItem->date().year(),curItem->date().month(),rx.cap(1).toInt());
+									matches[date].push_back(curItem->metaContact());
+								}
+							}
+						}
+						else
+						{
+							kDebug(14310) << "Error: Cannot parse:" << msgItem;
+						}
+					}
+					qApp->processEvents();
+					if (!mSearching) return;
+				}
 				file.close();
 			}
 		}
-	}
 
-	if ( item && mSearch->dateSearchMap[item->date()].contains(item->metaContact()) )
-		item->setHidden(false);
+		// relevant logfiles have been searched now, check if current date matches
+		if (matches[curItem->date()].contains(curItem->metaContact()))
+			curItem->setHidden(false);
 
-	mSearch->itemIndex++;
-
-	if( mSearch->itemIndex < mMainWidget->dateTreeWidget->topLevelItemCount() )
-	{
-		// Next iteration
+		// Next date item
 		mMainWidget->searchProgress->setValue(mMainWidget->searchProgress->value()+1);
-
-		QTimer::singleShot(0,this,SLOT(searchFirstStep()));
 	}
-	else
-	{
-		mMainWidget->searchButton->setText(i18n("&Search"));
+	searchFinished();
+}
 
-		delete mSearch;
-		mSearch = 0L;
-		doneProgressBar();
-	}
+void HistoryDialog::searchFinished()
+{
+	mMainWidget->searchButton->setText(i18n("&Search"));
+	mSearching = false;
+	doneProgressBar();
 }
 
 
@@ -611,4 +634,64 @@ void HistoryDialog::slotCopyURL()
 	connect( QApplication::clipboard(), SIGNAL( selectionChanged()), mHtmlPart, SLOT(slotClearSelection()));
 }
 
+void HistoryDialog::slotImportHistory(void)
+{
+	HistoryImport importer(this);
+	importer.exec();
+}
+
+QString HistoryDialog::highlight(const QString &htmlText, const QString &highlight) const
+{
+	const int highlightLength = highlight.length();
+	QString highlightedText;
+	int eIndex = -1;
+	int sIndex = 0;
+	int midLen;
+
+	for (;;)
+	{
+		sIndex = htmlText.indexOf("<", eIndex + 1);
+		midLen = (sIndex == -1) ? -1 : sIndex - eIndex - 1;
+
+		//Text to highlight
+		QString body = htmlText.mid(eIndex + 1, midLen);
+		int highlightIndex = 0;
+		while ((highlightIndex = body.indexOf(highlight, highlightIndex, Qt::CaseInsensitive)) > -1)
+		{
+			QString after = QString("<span style=\"background-color:yellow\">%1</span>").arg(body.mid(highlightIndex, highlightLength));
+			body.replace(highlightIndex, highlightLength, after);
+			highlightIndex += after.length();
+		}
+		highlightedText += body;
+		
+		if (sIndex == -1)
+			break;
+
+		eIndex = htmlText.indexOf(">", sIndex);
+		midLen = (eIndex == -1) ? -1 : eIndex - sIndex + 1;
+		highlightedText += htmlText.mid(sIndex, midLen); // Tag element
+
+		if (eIndex == -1)
+			break;
+	}
+
+	return highlightedText;
+}
+
+QString HistoryDialog::escapeXMLText(const QString& text) const
+{
+	if (text.isEmpty())
+		return QString();
+
+	QDomDocument doc;
+	QDomElement tmpElement = doc.createElement("tmpElement");
+	QDomText tmpTextNode = doc.createTextNode(text);
+	tmpElement.appendChild(tmpTextNode); //We have to attach the text to element otherwise save won't work correctly
+
+	QString excapedText;
+	QTextStream stream(&excapedText, QIODevice::WriteOnly);
+	tmpTextNode.save(stream, 0);
+	return excapedText;
+}
+
 #include "historydialog.moc"
