@@ -148,6 +148,9 @@ void AIMMyselfContact::sendMessage( Kopete::Message& message, Kopete::ChatSessio
 	s.replace ( QRegExp( QString::fromLatin1("<style>([^\"]*)font-size:([^p]*)pt;([^\"]*)\"([^<]*)</style>")),
 			QString::fromLatin1("<font ptsize=\"\\2\"><style>\\1\\3\"\\4</style></font>"));
 
+	s.replace ( QRegExp( QString::fromLatin1("<style>([^\"]*)background-color:([^;]*);([^\"]*)\"([^<]*)</style>")),
+	            QString::fromLatin1("<font back=\"\\2\"><style>\\1\\3\"\\4</style></font>"));
+
 	s.replace ( QRegExp( QString::fromLatin1("<style>([^\"]*)color:([^;]*);([^\"]*)\"([^<]*)</style>")),
 			QString::fromLatin1("<font color=\"\\2\"><style>\\1\\3\"\\4</style></font>"));
 
@@ -351,14 +354,14 @@ void AIMAccount::setOnlineStatus( const Kopete::OnlineStatus& status, const Kope
 		Oscar::Presence pres = protocol()->statusManager()->presenceOf( status );
 		if ( options & Kopete::Account::KeepSpecialFlags )
 			pres.setFlags( presence().flags() );
-		
+
 		setPresenceTarget( pres, reason.message() );
 	}
 }
 
 void AIMAccount::setStatusMessage( const Kopete::StatusMessage& statusMessage )
 {
-	Q_UNUSED(statusMessage);
+	setOnlineStatus( myself()->onlineStatus(), statusMessage, Kopete::Account::KeepSpecialFlags );
 }
 
 void AIMAccount::setUserProfile(const QString &profile)
@@ -450,7 +453,7 @@ void AIMAccount::disconnected( DisconnectReason reason )
 	myself()->setOnlineStatus( protocol()->statusManager()->onlineStatusOf( pres ) );
 
 	QHash<QString, Kopete::Contact*> contactList = contacts();
-	foreach( Kopete::Contact* c, contactList.values() )
+	foreach( Kopete::Contact* c, contactList )
 	{
 		OscarContact* oc = dynamic_cast<OscarContact*>( c );
 		if ( oc )
@@ -472,7 +475,8 @@ void AIMAccount::messageReceived( const Oscar::Message& message )
 		// Might be duplicate code from the parent class to get some needed information
 		// Perhaps a refactoring is needed.
 		kDebug(14152) << "Checking to see if I'm online..";
-		if( myself()->onlineStatus().status() == Kopete::OnlineStatus::Away )
+		if( myself()->onlineStatus().status() == Kopete::OnlineStatus::Away ||
+			myself()->onlineStatus().status() == Kopete::OnlineStatus::Busy)
 		{
 			QString sender = Oscar::normalize( message.sender() );
 			AIMContact* aimSender = dynamic_cast<AIMContact *> ( contacts().value( sender ) ); //should exist now
@@ -655,10 +659,9 @@ void AIMAccount::connectWithPassword( const QString &password )
 		Oscar::DWORD status = protocol()->statusManager()->oscarStatusOf( pres );
 		updateVersionUpdaterStamp();
 
-		Connection* c = setupConnection();
 		engine()->start( server, port, accountId(), password.left(16) );
 		engine()->setStatus( status, mInitialStatusMessage );
-		engine()->connectToServer( c, server, port, true /* doAuth */ );
+		engine()->connectToServer( server, port );
 
 		mInitialStatusMessage.clear();
 	}
