@@ -264,7 +264,8 @@ void ICQContact::loggedIn()
 void ICQContact::slotRequestAuth()
 {
 	QString reason = KInputDialog::getText( i18n("Request Authorization"),
-	                                        i18n("Reason for requesting authorization:") );
+	                                        i18n("Reason for requesting authorization:"),
+	                                        i18n("Please authorize me so I can add you to my contact list") );
 	if ( !reason.isNull() )
 		mAccount->engine()->requestAuth( contactId(), reason );
 }
@@ -627,6 +628,13 @@ QList<KAction*> *ICQContact::customContextMenuActions()
 
 void ICQContact::slotUserInfo()
 {
+	if ( m_infoWidget )
+	{
+		m_infoWidget->showNormal();
+		m_infoWidget->activateWindow();
+		return;
+	}
+
 	m_infoWidget = new ICQUserInfoWidget( this, Kopete::UI::Global::mainWidget() );
 	QObject::connect( m_infoWidget, SIGNAL(finished()), this, SLOT(closeUserInfoDialog()) );
 	QObject::connect( m_infoWidget, SIGNAL(okClicked()), this, SLOT(storeUserInfoDialog()) );
@@ -636,9 +644,10 @@ void ICQContact::slotUserInfo()
 void ICQContact::storeUserInfoDialog()
 {
 	QString alias = m_infoWidget->getAlias();
-	mAccount->engine()->changeContactAlias( contactId(), alias );
 	if ( alias.isEmpty() )
 		requestShortInfoDelayed( 5000 );
+	else
+		mAccount->engine()->changeContactAlias( contactId(), alias );
 }
 
 void ICQContact::closeUserInfoDialog()
