@@ -30,13 +30,12 @@
     #define error(parent, message, caption) \
         critical(parent, caption, message)
 #else
-    #include "mainwindow.h"
     #include "settings.h"
     #include <KActionCollection>
+    #include <KMainWindow>
     #include <KMessageBox>
     #include <KPasswordDialog>
-
-    #include <QAction>
+    #include <KXMLGUIClient>
 #endif
 
 #include <QApplication>
@@ -55,7 +54,6 @@ VncView::VncView(QWidget *parent, const KUrl &url, KConfigGroup configGroup)
         : RemoteView(parent),
         m_initDone(false),
         m_buttonMask(0),
-        m_modifiersMask(0),
         m_repaint(false),
         m_quitFlag(false),
         m_firstPasswordTry(true),
@@ -80,17 +78,13 @@ VncView::VncView(QWidget *parent, const KUrl &url, KConfigGroup configGroup)
     
 #ifndef QTONLY
     m_hostPreferences = new VncHostPreferences(configGroup, this);
+#else
+    Q_UNUSED(configGroup);
 #endif
 }
 
 VncView::~VncView()
 {
-    // Disconnect all signals so that we don't get any more callbacks from the client thread
-    disconnect(&vncThread, SIGNAL(imageUpdated(int, int, int, int)), this, SLOT(updateImage(int, int, int, int)));
-    disconnect(&vncThread, SIGNAL(gotCut(const QString&)), this, SLOT(setCut(const QString&)));
-    disconnect(&vncThread, SIGNAL(passwordRequest()), this, SLOT(requestPassword()));
-    disconnect(&vncThread, SIGNAL(outputErrorMessage(QString)), this, SLOT(outputErrorMessage(QString)));
-
     startQuitting();
 }
 
@@ -161,19 +155,31 @@ void VncView::startQuitting()
 {
     kDebug(5011) << "about to quit";
 
-    const bool connected = status() == RemoteView::Connected;
-
     setStatus(Disconnecting);
 
+    unpressModifiers();
+
+    // Disconnect all signals so that we don't get any more callbacks from the client thread
+    bool imageUpdatedDisconnect = disconnect(&vncThread, SIGNAL(imageUpdated(int, int, int, int)), this, SLOT(updateImage(int, int, int, int)));
+    bool gotCutDisconnect = disconnect(&vncThread, SIGNAL(gotCut(const QString&)), this, SLOT(setCut(const QString&)));
+    bool passwordRequestDisconnect = disconnect(&vncThread, SIGNAL(passwordRequest()), this, SLOT(requestPassword()));
+    bool outputErrorMessageDisconnect = disconnect(&vncThread, SIGNAL(outputErrorMessage(QString)), this, SLOT(outputErrorMessage(QString)));
+
+    kDebug(5011) << "Signals disconnected: imageUpdated: " << imageUpdatedDisconnect << "gotCut: " << gotCutDisconnect << "passwordRequest: " << passwordRequestDisconnect << "outputErrorMessage: " << outputErrorMessageDisconnect;
+
+    const bool connected = status() == RemoteView::Connected;
+
     m_quitFlag = true;
 
     if (connected) {
         vncThread.stop();
-    } else {
-        vncThread.quit();
     }
 
-    vncThread.wait(500);
+    vncThread.quit();
+
+    const bool quitSuccess = vncThread.wait(1000);
+
+    kDebug(5011) << "Quit VNC thread success:" << quitSuccess;
 
     setStatus(Disconnected);
 }
@@ -202,9 +208,9 @@ bool VncView::start()
         showDotCursor(RemoteView::CursorOn);
 #ifndef QTONLY
         // KRDC does always just have one main window, so at(0) is safe
-        MainWindow *mainWindow = qobject_cast<MainWindow*>(KMainWindow::memberList().at(0));
+        KXMLGUIClient *mainWindow = dynamic_cast<KXMLGUIClient*>(KMainWindow::memberList().at(0));
         if (mainWindow)
-            mainWindow->mainWindowActionCollection()->action("show_local_cursor")->setChecked(true);
+            mainWindow->actionCollection()->action("show_local_cursor")->setChecked(true);
 #endif
     }
 
@@ -236,11 +242,13 @@ void VncView::requestPassword()
     setStatus(Authenticating);
 
 #ifndef QTONLY
-    if (m_hostPreferences->walletSupport()) {
+    // just try to get the passwort from the wallet the first time, otherwise it will loop (see issue #226283)
+    if (m_firstPasswordTry && m_hostPreferences->walletSupport()) {
         QString walletPassword = readWalletPassword();
 
         if (!walletPassword.isNull()) {
             vncThread.setPassword(walletPassword);
+            m_firstPasswordTry = false;
             return;
         }
     }
@@ -330,14 +338,19 @@ void VncView::updateImage(int x, int y, int w, int h)
         setMouseTracking(true); // get mouse events even when there is no mousebutton pressed
         setFocusPolicy(Qt::WheelFocus);
         setStatus(Connected);
-//         emit framebufferSizeChanged(m_frame.width(), m_frame.height());
         emit connected();
         
         if (m_scale) {
-            if (parentWidget())
-                scaleResize(parentWidget()->width(), parentWidget()->height());
-        } 
-        
+#ifndef QTONLY
+            kDebug(5011) << "Setting initial size w:" <<m_hostPreferences->width() << " h:" << m_hostPreferences->height();
+            emit framebufferSizeChanged(m_hostPreferences->width(), m_hostPreferences->height());
+            scaleResize(m_hostPreferences->width(), m_hostPreferences->height());
+            kDebug() << "m_frame.size():" << m_frame.size() << "size()" << size();
+#else
+//TODO: qtonly alternative
+#endif
+        }
+
         m_initDone = true;
 
 #ifndef QTONLY
@@ -351,13 +364,15 @@ void VncView::updateImage(int x, int y, int w, int h)
         kDebug(5011) << "Updating framebuffer size";
         if (m_scale) {
             setMaximumSize(QSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));
+            if (parentWidget())
+                scaleResize(parentWidget()->width(), parentWidget()->height());
         } else {
             kDebug(5011) << "Resizing: " << m_frame.width() << m_frame.height();
             resize(m_frame.width(), m_frame.height());
             setMaximumSize(m_frame.width(), m_frame.height()); //This is a hack to force Qt to center the view in the scroll area
             setMinimumSize(m_frame.width(), m_frame.height());
+            emit framebufferSizeChanged(m_frame.width(), m_frame.height());
         }
-        emit framebufferSizeChanged(m_frame.width(), m_frame.height());
     }
 
     m_repaint = true;
@@ -369,6 +384,8 @@ void VncView::setViewOnly(bool viewOnly)
 {
     RemoteView::setViewOnly(viewOnly);
 
+    m_dontSendClipboard = viewOnly;
+
     if (viewOnly)
         setCursor(Qt::ArrowCursor);
     else
@@ -526,94 +543,46 @@ void VncView::wheelEventHandler(QWheelEvent *event)
 
 void VncView::keyEventHandler(QKeyEvent *e)
 {
-    int mask = 0;
-    
-    rfbKeySym k = 0;
-    switch (e->key()) {
-    case Qt::Key_Backtab: k = XK_Tab; break;
-    case Qt::Key_Backspace: k = XK_BackSpace; break;
-    case Qt::Key_Tab: k = XK_Tab; break;
-    case Qt::Key_Clear: k = XK_Clear; break;
-    case Qt::Key_Return: k = XK_Return; break;
-    case Qt::Key_Pause: k = XK_Pause; break;
-    case Qt::Key_Escape: k = XK_Escape; break;
-    case Qt::Key_Space: k = XK_space; break;
-    case Qt::Key_Delete: k = XK_Delete; break;
-    case Qt::Key_Enter: k = XK_KP_Enter; break;
-    case Qt::Key_Equal: k = XK_equal; break;
-    case Qt::Key_Up: k = XK_Up; break;
-    case Qt::Key_Down: k = XK_Down; break;
-    case Qt::Key_Right: k = XK_Right; break;
-    case Qt::Key_Left: k = XK_Left; break;
-    case Qt::Key_Insert: k = XK_Insert; break;
-    case Qt::Key_Home: k = XK_Home; break;
-    case Qt::Key_End: k = XK_End; break;
-    case Qt::Key_PageUp: k = XK_Page_Up; break;
-    case Qt::Key_PageDown: k = XK_Page_Down; break;
-    case Qt::Key_F1: k = XK_F1; break;
-    case Qt::Key_F2: k = XK_F2; break;
-    case Qt::Key_F3: k = XK_F3; break;
-    case Qt::Key_F4: k = XK_F4; break;
-    case Qt::Key_F5: k = XK_F5; break;
-    case Qt::Key_F6: k = XK_F6; break;
-    case Qt::Key_F7: k = XK_F7; break;
-    case Qt::Key_F8: k = XK_F8; break;
-    case Qt::Key_F9: k = XK_F9; break;
-    case Qt::Key_F10: k = XK_F10; break;
-    case Qt::Key_F11: k = XK_F11; break;
-    case Qt::Key_F12: k = XK_F12; break;
-    case Qt::Key_F13: k = XK_F13; break;
-    case Qt::Key_F14: k = XK_F14; break;
-    case Qt::Key_F15: k = XK_F15; break;
-    case Qt::Key_NumLock: k = XK_Num_Lock; break;
-    case Qt::Key_CapsLock: k = XK_Caps_Lock; break;
-    case Qt::Key_ScrollLock: k = XK_Scroll_Lock; break;
-    case Qt::Key_Shift: k = XK_Shift_L; mask |= KMOD_Shift_L; break;
-    case Qt::Key_Control: k = XK_Control_L; mask |= KMOD_Control_L; break;
-    case Qt::Key_AltGr: k = XK_ISO_Level3_Shift; mask |= KMOD_Alt_R; break;
-    case Qt::Key_Alt: k = XK_Alt_L; mask |= KMOD_Alt_L; break;
-    case Qt::Key_Meta: k = XK_Meta_L; mask |= KMOD_Meta_L; break;
-    case Qt::Key_Mode_switch: k = XK_Mode_switch; break;
-    case Qt::Key_Help: k = XK_Help; break;
-    case Qt::Key_Print: k = XK_Print; break;
-    case Qt::Key_SysReq: k = XK_Sys_Req; break;
-    default: break;
-    }
-    
-    // Transform dead keys
-    if (e->key() >= Qt::Key_Dead_Grave && e->key() <= Qt::Key_Dead_Horn) {
-        k = e->key() - Qt::Key_Dead_Grave + XK_dead_grave;
-    }
-    
-    const bool pressed = (e->type() == QEvent::KeyPress) ? true : false;
-    m_modifiersMask = pressed ? m_modifiersMask | mask : m_modifiersMask & ~mask;
+    // strip away autorepeating KeyRelease; see bug #206598
+    if (e->isAutoRepeat() && (e->type() == QEvent::KeyRelease))
+        return;
 
-    const bool hasShift = m_modifiersMask & KMOD_Shift_L;
-    const bool hasOtherMod = m_modifiersMask & (KMOD_Alt_R | KMOD_Alt_L | KMOD_Meta_L | KMOD_Control_L);
-    const bool isUpper = e->key() >= 'A' && e->key() <= 'Z';
-    // bool isLower = e->key() >= 'a' && e->key() <= 'z';
-    // bool isLetter = isLower || isUpper;
-    
-    // If shift is pressed, we receive Alt as Meta, but we want to actually send Alt.
-    if (k == XK_Meta_L && hasShift) k = XK_Alt_L;
+// parts of this code are based on http://italc.sourcearchive.com/documentation/1.0.9.1/vncview_8cpp-source.html
+    rfbKeySym k = e->nativeVirtualKey();
 
-    if (k == 0) {
-        if (hasOtherMod && (! isUpper || hasShift)) {
-            k = e->key();
+    // we do not handle Key_Backtab separately as the Shift-modifier
+    // is already enabled
+    if (e->key() == Qt::Key_Backtab) {
+        k = XK_Tab;
+    }
+
+    const bool pressed = (e->type() == QEvent::KeyPress);
+
+    // handle modifiers
+    if (k == XK_Shift_L || k == XK_Control_L || k == XK_Meta_L || k == XK_Alt_L) {
+        if (pressed) {
+            m_mods[k] = true;
+        } else if (m_mods.contains(k)) {
+            m_mods.remove(k);
         } else {
-            if (e->key() < 0x100 && e->text().length() > 0)
-                k = QChar(e->text().at(0)).unicode(); //respect upper- / lowercase
-            else
-                rfbClientLog("Unknown keysym: 0x%x\n", e->key());
+            unpressModifiers();
         }
     }
 
-    if (k < 26) // workaround for modified keys by pressing CTRL
-        k += 96;
+    if (k) {
+        vncThread.keyEvent(k, pressed);
+    }
+}
 
-    //rfbClientLog("Key event(%s): orig: 0x%x, sent: 0x%x\n", pressed ? "P" : "R", e->key(), k);
-
-    vncThread.keyEvent(k, pressed);
+void VncView::unpressModifiers()
+{
+    const QList<unsigned int> keys = m_mods.keys();
+    QList<unsigned int>::const_iterator it = keys.constBegin();
+    while (it != keys.end()) {
+        vncThread.keyEvent(*it, false);
+        it++;
+    }
+    m_mods.clear();
 }
 
 void VncView::clipboardSelectionChanged()
@@ -646,6 +615,4 @@ void VncView::clipboardDataChanged()
     vncThread.clientCut(text);
 }
 
-#ifndef QTONLY
 #include "moc_vncview.cpp"
-#endif
